@@ -127,8 +127,10 @@ Sheet		*excel_externsheet_v7	 (MSContainer const *container, gint16 i);
 ExcelExternSheetV8 const *excel_externsheet_v8 (ExcelWorkbook const *wb, gint16 i);
 
 void		excel_read_EXTERNSHEET_v7 (BiffQuery const *q, MSContainer *container);
-MsBiffBofData  *ms_biff_bof_data_new     (BiffQuery * q);
-void		ms_biff_bof_data_destroy (MsBiffBofData * data);
+MsBiffBofData *ms_biff_bof_data_new     (BiffQuery * q);
+void	       ms_biff_bof_data_destroy (MsBiffBofData * data);
+char	      *ms_biff_get_chars (char const *ptr, guint length,
+				  gboolean use_utf16);
 
 StyleColor  *excel_palette_get (ExcelPalette const *pal, gint idx);
 
