@@ -6,7 +6,7 @@
  *    Michael Meeks (michael@ximian.com)
  *    Jody Goldberg (jody@gnome.org)
  *
- * (C) 1998-2002 Michael Meeks, Jody Goldberg
+ * (C) 1998-2003 Michael Meeks, Jody Goldberg
  */
 #include <gnumeric-config.h>
 #include <gnumeric-i18n.h>
@@ -47,8 +47,7 @@
  *     macrofun.hlp has info on them but supporting Excel4 macro sheets is not
  *     top priority.
  **/
-FormulaFuncData const formula_func_data[FORMULA_FUNC_DATA_LEN] =
-{
+MsFormulaFuncData const formula_func_data[FORMULA_FUNC_DATA_LEN] = {
 /* 0 */		{ "COUNT", -1 },
 /* 1 */		{ "IF", -1 },
 /* 2 */		{ "ISNA", 1 },
@@ -409,7 +408,7 @@ FormulaFuncData const formula_func_data[FORMULA_FUNC_DATA_LEN] =
 /* 357 */	{ "VIEWGET", -2 },
 /* 358 */	{ "GETPIVOTDATA", 2 },
 /* 359 */	{ "HYPERLINK", -1 },	/* cell_contents is optional */
-/* 360 */	{ "PHONETIC", -2 },
+/* 360 */	{ "PHONETIC", 1 },
 /* 361 */	{ "AVERAGEA", -1 },
 /* 362 */	{ "MAXA", -1 },
 /* 363 */	{ "MINA", -1 },
@@ -530,11 +529,16 @@ getRefV8 (CellRef *cr,
 static void
 parse_list_push (GnmExprList **list, GnmExpr const *pd)
 {
-	d (5, fprintf (stderr, "Push 0x%x\n", (int)pd););
-	if (!pd)
-		fprintf (stderr, "FIXME: Pushing nothing onto excel function stack\n");
+	d (5, fprintf (stderr, "Push 0x%p\n", pd););
+	if (pd == NULL) {
+		g_warning ("FIXME: Pushing nothing onto excel function stack");
+		pd = expr_tree_error (NULL, -1, -1,
+			"Incorrect number of parsed formula arguments",
+			"#WrongArgs");
+	}
 	*list = gnm_expr_list_prepend (*list, pd);
 }
+
 static void
 parse_list_push_raw (GnmExprList **list, Value *v)
 {
@@ -608,7 +612,7 @@ make_function (GnmExprList **stack, int fn_idx, int numargs)
 			parse_list_free (&args);
 			parse_list_push_raw (stack,
 				value_new_error (NULL, _("Broken function")));
-			fprintf (stderr, "So much for that theory.\n");
+			g_warning ("So much for that theory.");
 			return FALSE;
 		}
 
@@ -621,7 +625,7 @@ make_function (GnmExprList **stack, int fn_idx, int numargs)
 		parse_list_push (stack, gnm_expr_new_funcall (name, args));
 		return TRUE;
 	} else if (fn_idx >= 0 && fn_idx < FORMULA_FUNC_DATA_LEN) {
-		FormulaFuncData const *fd = &formula_func_data [fn_idx];
+		MsFormulaFuncData const *fd = &formula_func_data [fn_idx];
 		GnmExprList *args;
 
 		d (2, fprintf (stderr, "Function '%s', args %d, templ: %d\n",
@@ -647,14 +651,14 @@ make_function (GnmExprList **stack, int fn_idx, int numargs)
 		if (fd->name) {
 			name = gnm_func_lookup (fd->name, NULL);
 			if (name == NULL)
-				name = gnm_func_add_placeholder (fd->name, "Builtin ", FALSE);
+				name = gnm_func_add_placeholder (fd->name, "Builtin ", TRUE);
 		}
 		/* This should not happen */
 		if (!name) {
 			char *txt;
 			txt = g_strdup_printf ("[Function '%s']",
 					       fd->name ? fd->name : "?");
-			fprintf (stderr, "Unknown %s\n", txt);
+			g_warning ("Unknown %s", txt);
 			parse_list_push_raw (stack, value_new_error (NULL, txt));
 			g_free (txt);
 
@@ -664,7 +668,7 @@ make_function (GnmExprList **stack, int fn_idx, int numargs)
 		parse_list_push (stack, gnm_expr_new_funcall (name, args));
 		return TRUE;
 	} else
-		fprintf (stderr, "FIXME, unimplemented fn 0x%x, with %d args\n",
+		g_warning ("FIXME, unimplemented fn 0x%x, with %d args",
 			fn_idx, numargs);
 	return FALSE;
 }
@@ -1070,7 +1074,7 @@ excel_parse_formula (MSContainer const *container,
 
 		case FORMULA_PTG_EXTENDED : { /* Extended Ptgs for Biff8 */
 			/*
-			 * The beginings of 'extended' ptg support.
+			 * The beginnings of 'extended' ptg support.
 			 * These are mostly undocumented.
 			 */
 			/* Use 0 for unknown sizes, and ignore the trailing
