@@ -27,12 +27,14 @@
 #include <goffice/graph/gog-data-allocator.h>
 #include <goffice/graph/gog-view.h>
 #include <goffice/graph/gog-chart.h>
+#include <goffice/graph/gog-plot.h>
 #include <goffice/graph/gog-renderer.h>
 #include <goffice/graph/go-data.h>
 
 #include <gsf/gsf-impl-utils.h>
 #include <src/gnumeric-i18n.h>
 #include <src/gui-util.h>
+#include <src/mathfunc.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtkcheckbutton.h>
 #include <glade/glade-xml.h>
@@ -55,7 +57,10 @@ struct _GogAxis {
 	GogDatasetElement source [AXIS_ELEM_LAST_ENTRY];
 	GogAxisTickLevel  tick_level;
 
-	double	min_val, max_val;
+	double		bound_min, bound_max, bound_step;
+	double		min_val, max_val;
+	double		logical_min_val, logical_max_val;
+	gpointer	min_contrib, max_contrib; /* NULL means use the manual sources */
 };
 
 typedef GogStyledObjectClass GogAxisClass;
@@ -140,6 +145,8 @@ gog_axis_finalize (GObject *obj)
 {
 	GogAxis *axis = GOG_AXIS (obj);
 
+	gog_axis_clear_contributors (axis);
+
 	g_slist_free (axis->i_cross);	 	axis->i_cross = NULL;
 	g_slist_free (axis->crosses_me); 	axis->crosses_me = NULL;
 	g_slist_free (axis->contributors);	axis->contributors = NULL;
@@ -154,11 +161,61 @@ gog_axis_update (GogObject *obj)
 {
 	GSList *ptr;
 	GogAxis *axis = GOG_AXIS (obj);
+	double minima, maxima, logical_min, logical_max;
+	double range, step;
+	double old_min = axis->bound_min;
+	double old_max = axis->bound_max;
 
-	g_warning ("axis::update");
+	gog_debug (0, g_warning ("axis::update"););
 
+	axis->min_val = DBL_MAX;
+	axis->max_val = DBL_MIN;
+	axis->min_contrib = axis->max_contrib = NULL;
 	for (ptr = axis->contributors ; ptr != NULL ; ptr = ptr->next) {
+		gog_plot_get_axis_bounds (GOG_PLOT (ptr->data), axis->type,
+			&minima, &maxima, &logical_min, &logical_max);
+
+		if (axis->min_val > minima) {
+			axis->min_val = minima;
+			axis->logical_min_val = logical_min;
+			axis->min_contrib = ptr->data;
+		} else if (axis->min_contrib == ptr->data) {
+			axis->min_contrib = NULL;
+			axis->min_val = minima;
+		}
+
+		if (axis->max_val < maxima) {
+			axis->max_val = maxima;
+			axis->logical_max_val = logical_max;
+			axis->max_contrib = ptr->data;
+		} else if (axis->max_contrib == ptr->data) {
+			axis->max_contrib = NULL;
+			axis->max_val = maxima;
+		}
 	}
+
+	minima = axis->min_val;
+	maxima = axis->max_val;
+	if (minima < maxima) {
+		range = fabs (maxima - minima);
+		if (gnumeric_sub_epsilon (range) < 0.) {
+			minima *= .9;
+			maxima *= 1.1;
+			range = fabs (maxima - minima);
+		}
+		step  = pow (10, gnumeric_fake_trunc (log10 (range)));
+		if (range/step < 3)
+			step /= 5.;
+
+		/* we want the bounds to be loose so jump up a step if we get too close */
+		axis->bound_min = step * floor (gnumeric_sub_epsilon (minima/step));
+		axis->bound_max = step * ceil (gnumeric_add_epsilon (maxima/step));
+		axis->bound_step = step;
+	} else
+		axis->bound_min = axis->bound_max = axis->bound_step = 0.;
+
+	if (old_min != axis->bound_min || old_max != axis->bound_max)
+		gog_object_emit_changed (GOG_OBJECT (obj), TRUE);
 }
 
 static void
@@ -216,26 +273,35 @@ gog_axis_editor (GogObject *gobj, GogDataAllocator *dalloc, CommandContext *cc)
 	gtk_notebook_prepend_page (GTK_NOTEBOOK (notebook),
 		glade_xml_get_widget (gui, "axis_pref_table"),
 		gtk_label_new (_("Details")));
-	gtk_notebook_prepend_page (GTK_NOTEBOOK (notebook),
-		gog_style_editor (gobj, cc, GOG_STYLE_LINE),
-		gtk_label_new (_("Style")));
+	gog_style_editor (gobj, cc, notebook, GOG_STYLE_LINE),
 
 	w = glade_xml_get_widget (gui, "axis_low"),
 	g_signal_connect_object (G_OBJECT (w),
 		"toggled",
 		G_CALLBACK (cb_pos_changed), axis, 0);
+
+	g_object_set_data_full (G_OBJECT (notebook), "gui", gui,
+				(GDestroyNotify)g_object_unref);
+
 	gtk_widget_show (GTK_WIDGET (notebook));
 	return notebook;
 }
+static unsigned
+gog_axis_interesting_fields (GogStyledObject *obj)
+{
+	return GOG_STYLE_LINE | GOG_STYLE_FONT;
+}
+
 static void
 gog_axis_class_init (GObjectClass *gobject_klass)
 {
 	static GogObjectRole const roles[] = {
-		{ N_("Label"), "GogLabel",
+		{ N_("Label"), "GogLabel", 0,
 		  GOG_POSITION_COMPASS, GOG_POSITION_N|GOG_POSITION_ALIGN_CENTER, GOG_OBJECT_NAME_BY_ROLE,
 		  NULL, NULL, NULL, NULL, NULL, NULL },
 	};
 	GogObjectClass *gog_klass = (GogObjectClass *) gobject_klass;
+	GogStyledObjectClass *style_klass = (GogStyledObjectClass *) gog_klass;
 
 	parent_klass = g_type_class_peek_parent (gobject_klass);
 	gobject_klass->set_property = gog_axis_set_property;
@@ -260,6 +326,7 @@ gog_axis_class_init (GObjectClass *gobject_klass)
 	gog_klass->update	= gog_axis_update;
 	gog_klass->editor	= gog_axis_editor;
 	gog_klass->view_type	= gog_axis_view_get_type ();
+	style_klass->interesting_fields = gog_axis_interesting_fields;
 }
 
 static void
@@ -269,6 +336,11 @@ gog_axis_init (GogAxis *axis)
 	axis->pos	 = GOG_AXIS_AT_LOW;
 	axis->tick_level = AXIS_ELEM_MINOR_TICK,
 	axis->i_cross =	axis->crosses_me = axis->contributors = NULL;
+
+	/* yes we want min = MAX */
+	axis->min_val = DBL_MAX;
+	axis->max_val = DBL_MIN;
+	axis->min_contrib = axis->max_contrib = NULL;
 }
 
 static void
@@ -287,11 +359,18 @@ gog_axis_dataset_get_elem (GogDataset const *set, int dim_i)
 	return NULL;
 }
 
+static void
+gog_axis_dim_changed (GogDataset *set, int dim_i)
+{
+	gog_object_emit_changed (GOG_OBJECT (set), TRUE);
+}
+
 static void
 gog_axis_dataset_init (GogDatasetClass *iface)
 {
-	iface->dims	= gog_axis_dataset_dims;
-	iface->get_elem	= gog_axis_dataset_get_elem;
+	iface->dims	   = gog_axis_dataset_dims;
+	iface->get_elem	   = gog_axis_dataset_get_elem;
+	iface->dim_changed = gog_axis_dim_changed;
 }
 
 GSF_CLASS_FULL (GogAxis, gog_axis,
@@ -301,19 +380,30 @@ GSF_CLASS_FULL (GogAxis, gog_axis,
 
 
 GogAxisType
-gog_axis_type (GogAxis const *axis)
+gog_axis_get_atype (GogAxis const *axis)
 {
 	g_return_val_if_fail (GOG_AXIS (axis) != NULL, GOG_AXIS_UNKNOWN);
 	return axis->type;
 }
 
 GogAxisPosition
-gog_axis_pos (GogAxis const *axis)
+gog_axis_get_pos (GogAxis const *axis)
 {
 	g_return_val_if_fail (GOG_AXIS (axis) != NULL, GOG_AXIS_IN_MIDDLE);
 	return axis->pos;
 }
 
+gboolean
+gog_axis_get_bounds (GogAxis const *axis, double *min_bound, double *max_bound)
+{
+	g_return_val_if_fail (GOG_AXIS (axis) != NULL, FALSE);
+
+	*min_bound = axis->bound_min;
+	*max_bound = axis->bound_max;
+
+	return TRUE;
+}
+
 /**
  * gog_axis_add_contributor :
  * @axis : #GogAxis
@@ -328,6 +418,8 @@ gog_axis_add_contributor (GogAxis *axis, GogObject *contrib)
 	g_return_if_fail (g_slist_find (axis->contributors, contrib) == NULL);
 
 	axis->contributors = g_slist_prepend (axis->contributors, contrib);
+
+	gog_object_request_update (GOG_OBJECT (axis));
 }
 
 /**
@@ -340,41 +432,74 @@ gog_axis_add_contributor (GogAxis *axis, GogObject *contrib)
 void
 gog_axis_del_contributor (GogAxis *axis, GogObject *contrib)
 {
+	gboolean update = FALSE;
+
 	g_return_if_fail (GOG_AXIS (axis) != NULL);
 	g_return_if_fail (g_slist_find (axis->contributors, contrib) != NULL);
 
+	if (axis->min_contrib == contrib) {
+		axis->min_contrib = NULL;
+		update = TRUE;
+	}
+	if (axis->max_contrib == contrib) {
+		axis->max_contrib = NULL;
+		update = TRUE;
+	}
 	axis->contributors = g_slist_remove (axis->contributors, contrib);
+
+	if (update)
+		gog_object_request_update (GOG_OBJECT (axis));
+}
+
+void
+gog_axis_clear_contributors (GogAxis *axis)
+{
+	GSList *ptr, *list;
+	GogAxisSet filter;
+
+	g_return_if_fail (GOG_AXIS (axis) != NULL);
+
+	filter = 1 << axis->type;
+	list = g_slist_copy (axis->contributors);
+	for (ptr = list; ptr != NULL ; ptr = ptr->next)
+		gog_plot_axis_clear (GOG_PLOT (ptr->data), filter);
+	g_slist_free (list);
+}
+
+GSList const *
+gog_axis_contributors (GogAxis *axis)
+{
+	g_return_val_if_fail (GOG_AXIS (axis) != NULL, NULL);
+
+	return axis->contributors;
 }
 
 /**
- *
-gog_axis_bound_changed   (GogAxis *axis, GogObject *contrib,
-double low, double high);
+ * gog_axis_bound_changed :
+ * @axis : #GogAxis
+ * @contrib : #GogObject
 **/
 void
-gog_axis_bound_changed (GogAxis *axis, GogObject *contrib,
-			double low, double high)
+gog_axis_bound_changed (GogAxis *axis, GogObject *contrib)
 {
 	g_return_if_fail (GOG_AXIS (axis) != NULL);
 
-	g_warning ("%p : bound changed to %g -> %g", axis, low, high);
 	gog_object_request_update (GOG_OBJECT (axis));
 }
 
 static unsigned
 gog_axis_num_markers (GogAxis *axis)
 {
-#warning bogus quicky
-	return 2;
+	if (axis->bound_step <= 0.)
+		return 0;
+
+	return 1 + fabs (axis->bound_max - axis->bound_min) / (double)axis->bound_step;
 }
 
-static char const *
+static char *
 gog_axis_get_marker (GogAxis *axis, unsigned i)
 {
-#warning bogus quicky
-	if (i == 0)
-		return "low";
-	return "high";
+	return g_strdup_printf ("%g", axis->bound_min + ((double)i) * axis->bound_step);
 }
 
 /****************************************************************************/
@@ -403,8 +528,9 @@ gog_axis_view_size_request (GogView *view, GogViewRequisition *req)
  * things are too big */
 	gog_renderer_push_style (view->renderer, axis->base.style);
 	for (i = gog_axis_num_markers (axis) ; i-- > 0 ; ) {
-		gog_renderer_measure_text (view->renderer,
-			gog_axis_get_marker (axis, i), &tmp);
+		char *txt = gog_axis_get_marker (axis, i);
+		gog_renderer_measure_text (view->renderer, txt, &tmp);
+		g_free (txt);
 		if (is_horiz) {
 			total += tmp.w;
 			if (max < tmp.h)
@@ -431,9 +557,12 @@ static void
 gog_axis_view_render (GogView *view, GogViewAllocation const *bbox)
 {
 	GogViewAllocation const *area = &view->residual;
+	GogViewRequisition size;
 	ArtVpath path[3];
+	ArtPoint pos;
 	GogAxis *axis = GOG_AXIS (view->model);
-	double pre, post;
+	unsigned n;
+	double pre, post, step, bound;
 	double line_width = gog_renderer_line_size (
 		view->renderer, axis->base.style->line.width) / 2;
 
@@ -441,6 +570,7 @@ gog_axis_view_render (GogView *view, GogViewAllocation const *bbox)
 
 	g_return_if_fail (axis->pos != GOG_AXIS_IN_MIDDLE);
 
+	gog_renderer_push_style (view->renderer, axis->base.style);
 	switch (axis->type) {
 	case GOG_AXIS_X:
 		gog_chart_view_get_indents (view->parent, &pre, &post);
@@ -461,18 +591,37 @@ gog_axis_view_render (GogView *view, GogViewAllocation const *bbox)
 		break;
 
 	case GOG_AXIS_Y:
-		path[0].y = area->y;
+		pos.y = path[0].y = area->y;
 		path[1].y = area->y + area->h;
 		switch (axis->pos) {
 		case GOG_AXIS_AT_LOW:
 			path[0].x = path[1].x = area->x + area->w - line_width;
+			pos.x = area->x;
 			break;
 		case GOG_AXIS_AT_HIGH:
 			path[0].x = path[1].x = area->x + line_width;
+			pos.x = area->x + area->w;
 			break;
 		default :
 			break;
 		}
+
+		n = gog_axis_num_markers (axis);
+		
+		if (n > 1)
+			step = area->h / (n - 1);
+		for (bound = -1 ; n-- > 0 ;) {
+			if (pos.y >= bound) {	/* keep thing sfrom overlapping */
+				char *txt = gog_axis_get_marker (axis, n);
+
+				size.w = area->w - line_width;
+				size.h = -1;
+				gog_renderer_draw_text (view->renderer, &pos, GTK_ANCHOR_W, txt, &size);
+				g_free (txt);
+				bound = pos.y + size.h;
+			}
+			pos.y += step;
+		}
 		break;
 	default :
 		break;
@@ -481,7 +630,6 @@ gog_axis_view_render (GogView *view, GogViewAllocation const *bbox)
 	path[0].code = ART_MOVETO;
 	path[1].code = ART_LINETO;
 	path[2].code = ART_END;
-	gog_renderer_push_style (view->renderer, axis->base.style);
 	gog_renderer_draw_path (view->renderer, path);
 	gog_renderer_pop_style (view->renderer);
 }
