@@ -43,12 +43,16 @@ typedef enum {
 
 GType gog_axis_get_type (void);
 
-void		gog_axis_add_contributor (GogAxis *axis, GogObject *contrib);
-void		gog_axis_del_contributor (GogAxis *axis, GogObject *contrib);
-void		gog_axis_bound_changed   (GogAxis *axis, GogObject *contrib,
-					  double low, double high);
-GogAxisType	gog_axis_type	 	 (GogAxis const *axis);
-GogAxisPosition gog_axis_pos	 	 (GogAxis const *axis);
+GogAxisType	gog_axis_get_atype 	 (GogAxis const *axis);
+GogAxisPosition gog_axis_get_pos 	 (GogAxis const *axis);
+gboolean	gog_axis_get_bounds 	 (GogAxis const *axis,
+					  double *min_val, double *max_val);
+
+void 	      gog_axis_add_contributor	  (GogAxis *axis, GogObject *contrib);
+void 	      gog_axis_del_contributor	  (GogAxis *axis, GogObject *contrib);
+GSList const *gog_axis_contributors	  (GogAxis *axis);
+void	      gog_axis_clear_contributors (GogAxis *axis);
+void	      gog_axis_bound_changed	  (GogAxis *axis, GogObject *contrib);
 
 G_END_DECLS
 
