@@ -49,6 +49,8 @@ gog_plot_finalize (GObject *obj)
 
 	g_slist_free (plot->series); /* GogObject does the unref */
 
+	gog_plot_axis_clear (plot, GOG_AXIS_SET_ALL); /* just in case */
+
 	if (parent_klass != NULL && parent_klass->finalize != NULL)
 		(parent_klass->finalize) (obj);
 }
@@ -161,7 +163,7 @@ static void
 gog_plot_class_init (GogObjectClass *gog_klass)
 {
 	static GogObjectRole const roles[] = {
-		{ N_("Series"), "GogSeries",
+		{ N_("Series"), "GogSeries",	0,
 		  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
 		  role_series_can_add, role_series_can_remove,
 		  role_series_allocate,
@@ -321,10 +323,16 @@ gog_plot_foreach_elem (GogPlot *plot, GogEnumFunc func, gpointer data)
 }
 
 GOData *
-gog_plot_get_axis_bounds (GogPlot *plot, unsigned axis,
-			  double *min, double *max)
+gog_plot_get_axis_bounds (GogPlot *plot, GogAxisType axis,
+			  double *min, double *max,
+			  double *logical_min, double *logical_max)
 {
-	return NULL;
+	GogPlotClass *klass = GOG_PLOT_GET_CLASS (plot);
+
+	g_return_val_if_fail (klass != NULL, FALSE);
+
+	return (klass->axis_bounds) (plot, axis, min, max,
+		logical_min, logical_max);
 }
 
 gboolean
@@ -393,6 +401,27 @@ gog_plot_axis_set_assign (GogPlot *plot, GogAxisSet axis_set)
 	return (klass->axis_set_assign) (plot, axis_set);
 }
 
+/**
+ * gog_plot_axis_clear : 
+ * @plot : #GogPlot
+ * @filter : #GogAxisSet
+ * 
+ * A utility method to clear all existing axis associations flagged by @filter
+ **/
+void
+gog_plot_axis_clear (GogPlot *plot, GogAxisSet filter)
+{
+	GogAxisType type;
+
+	g_return_if_fail (GOG_PLOT (plot) != NULL);
+
+	for (type = 0 ; type < GOG_AXIS_TYPES ; type++)
+		if (plot->axis[type] != NULL && ((1 << type) & filter)) {
+			gog_axis_del_contributor (plot->axis[type], GOG_OBJECT (plot));
+			plot->axis[type] = NULL;
+		}
+}
+
 /****************************************************************************/
 /* a placeholder.  It seems likely that we will want this eventually        */
 GSF_CLASS_ABSTRACT (GogPlotView, gog_plot_view,
