@@ -26,6 +26,7 @@
 #define GO_COLOR_H
 
 #include <glib.h>
+#include <goffice/utils/goffice-utils.h>
 #include <gtk/gtkwidget.h>
 #include <libart_lgpl/art_render.h>
 #include <libart_lgpl/art_svp.h>
@@ -102,8 +103,6 @@ UINT_TO_RGB((i), ((guchar*)p), ((guchar*)p)+1, ((guchar*)p)+2)
 #define PIXEL_GREY(p,g) PIXEL_RGB(p,g,g,g)
 #define PIXEL_GREYA(p,g,a) PIXEL_RGBA(p,g,g,g,a)
 
-typedef guint32	GOColor;
-
 void go_color_to_artpix  (ArtPixMaxDepth *res, GOColor rgba);
 void go_color_render_svp (GOColor color, ArtSVP const *svp,
 			  int x0, int y0, int x1, int y1,
@@ -112,6 +111,9 @@ void go_color_render_svp (GOColor color, ArtSVP const *svp,
 GOColor color_combo_get_gocolor (GtkWidget *cc); /* , GOColor default_val); */
 void    color_combo_set_gocolor (GtkWidget *cc, GOColor c);
 
+GOColor go_color_from_str (const gchar *string);
+gchar  *go_color_as_str (GOColor color);
+
 G_END_DECLS
 
 #endif /* GO_COLOR_H */
