@@ -992,41 +992,10 @@ dependent_add_dynamic_dep (Dependent *dep, ValueRange const *v)
 		workbook_link_3d_dep (dep);
 }
 
-static void
-dynamic_dep_free (DynamicDep *dyn)
-{
-	Dependent *dep = dyn->container;
-	CellPos const *pos = (dependent_is_cell (dep))
-		? &DEP_TO_CELL(dep)->pos : &dummy;
-	ValueRange *v;
-	GSList *ptr;
-
-	for (ptr = dyn->singles ; ptr != NULL ; ptr = ptr->next) {
-		v = ptr->data;
-		unlink_single_dep (&dyn->base, pos, &v->cell.a);
-		value_release ((Value *)v);
-	}
-	g_slist_free (dyn->singles);
-	dyn->singles = NULL;
-
-	for (ptr = dyn->ranges ; ptr != NULL ; ptr = ptr->next) {
-		v = ptr->data;
-		unlink_cellrange_dep (&dyn->base, pos, &v->cell.a, &v->cell.b);
-		value_release ((Value *)v);
-	}
-	g_slist_free (dyn->ranges);
-	dyn->ranges = NULL;
-
-	if (dyn->base.flags & DEPENDENT_HAS_3D)
-		workbook_unlink_3d_dep (&dyn->base);
-}
-
-static void
+static inline void
 dependent_clear_dynamic_deps (Dependent *dep)
 {
-	DynamicDep *dyn = g_hash_table_lookup (dep->sheet->deps->dynamic_deps, dep);
-	if (dyn != NULL)
-		dynamic_dep_free (dyn);
+	g_hash_table_remove (dep->sheet->deps->dynamic_deps, dep);
 }
 
 /*****************************************************************************/
@@ -1051,12 +1020,14 @@ dependent_link (Dependent *dep, CellPos const *pos)
 
 	sheet = dep->sheet;
 
-	/* Make this the new head of the dependent list.  */
-	dep->prev_dep = NULL;
-	dep->next_dep = sheet->deps->dependent_list;
-	if (dep->next_dep)
-		dep->next_dep->prev_dep = dep;
-	sheet->deps->dependent_list = dep;
+	/* Make this the new tail of the dependent list.  */
+	dep->prev_dep = sheet->deps->tail;
+	dep->next_dep = NULL;
+	if (dep->prev_dep)
+		dep->prev_dep->next_dep = dep;
+	else
+		sheet->deps->head = dep; /* first element */
+	sheet->deps->tail = dep;
 	dep->flags |=
 		DEPENDENT_IS_LINKED |
 		link_expr_dep (dep, pos, dep->expression);
@@ -1076,6 +1047,8 @@ dependent_link (Dependent *dep, CellPos const *pos)
 void
 dependent_unlink (Dependent *dep, CellPos const *pos)
 {
+	GnmDepContainer *contain;
+
 	g_return_if_fail (dep != NULL);
 	g_return_if_fail (dependent_is_linked (dep));
 	g_return_if_fail (dep->expression != NULL);
@@ -1085,9 +1058,12 @@ dependent_unlink (Dependent *dep, CellPos const *pos)
 		pos = (dependent_is_cell (dep)) ? &DEP_TO_CELL(dep)->pos : &dummy;
 
 	unlink_expr_dep (dep, pos, dep->expression);
-	if (dep->sheet->deps != NULL) {
-		if (dep->sheet->deps->dependent_list == dep)
-			dep->sheet->deps->dependent_list = dep->next_dep;
+	contain = dep->sheet->deps;
+	if (contain != NULL) {
+		if (contain->head == dep)
+			contain->head = dep->next_dep;
+		if (contain->tail == dep)
+			contain->tail = dep->prev_dep;
 		if (dep->next_dep)
 			dep->next_dep->prev_dep = dep->prev_dep;
 		if (dep->prev_dep)
@@ -1681,8 +1657,7 @@ dependents_relocate (GnmExprRelocateInfo const *info)
 		eval_pos_init_dep (&rwinfo.u.relocate.pos, dep);
 
 		/* it is possible nothing changed for contained deps
-		 * using absolute references
-		 */
+		 * using absolute references */
 		newtree = gnm_expr_rewrite (dep->expression, &rwinfo);
 		if (newtree != NULL) {
 			int const t = dependent_type (dep);
@@ -1776,17 +1751,21 @@ dependents_relocate (GnmExprRelocateInfo const *info)
 inline static void
 invalidate_refs (Dependent *dep, GnmExprRewriteInfo const *rwinfo)
 {
-	GnmExpr const *newtree = gnm_expr_rewrite (dep->expression, rwinfo);
+	/* Ignore any linger dynamic deps, they'll get cleared soon */
+	if (dependent_type (dep) != DEPENDENT_DYNAMIC_DEP) {
 
-	/* We are told this dependent depends on this region, hence if newtree
-	 * is null then either
-	 * 1) we did not depend on it ( ie. serious breakage )
-	 * 2j we had a duplicate reference and we have already removed it.
-	 * 3) We depended on things via a name which will be invalidated elsewhere
-	 */
-	if (newtree != NULL) {
-		dependent_set_expr (dep, newtree);
-		gnm_expr_unref (newtree);
+		/* We are told this dependent depends on this region, hence if
+		 * newtree is null then either
+		 * 1) we did not depend on it ( ie. serious breakage )
+		 * 2) we had a duplicate reference and we have already removed it.
+		 * 3) We depended on things via a name which will be
+		 *    invalidated elsewhere */
+		GnmExpr const *newtree =
+			gnm_expr_rewrite (dep->expression, rwinfo);
+		if (newtree != NULL) {
+			dependent_set_expr (dep, newtree);
+			gnm_expr_unref (newtree);
+		}
 	}
 }
 
@@ -1795,11 +1774,10 @@ invalidate_refs (Dependent *dep, GnmExprRewriteInfo const *rwinfo)
  * This is tightly coupled with do_deps_destroy.
  */
 static void
-cb_dep_hash_invalidate (G_GNUC_UNUSED gpointer key, gpointer value,
-			gpointer closure)
+cb_dep_hash_invalidate (G_GNUC_UNUSED gpointer key,
+			DependencyAny *depany,
+			GnmExprRewriteInfo const *rwinfo)
 {
-	GnmExprRewriteInfo const *rwinfo = closure;
-	DependencyAny *depany = value;
 #ifndef ENABLE_MICRO_HASH
 	GSList *deps = depany->deps;
 	GSList *ptr = deps;
@@ -1849,6 +1827,32 @@ cb_dep_hash_invalidate (G_GNUC_UNUSED gpointer key, gpointer value,
 #endif
 }
 
+static void
+cb_find_dynamic_deps (G_GNUC_UNUSED gpointer key,
+		      DependencyAny *depany, GSList **res)
+{
+	GSList *accum = *res;
+	dep_collection_foreach_dep (depany->deps, dep,
+		if (dependent_type (dep) == DEPENDENT_DYNAMIC_DEP)
+			accum = g_slist_prepend (accum, ((DynamicDep *)dep)->container););
+	*res = accum;
+}
+
+static void
+dep_hash_destroy (GHashTable *hash, GnmExprRewriteInfo const *rwinfo,
+		  GSList **extern_dyn_dep)
+{
+	/* Collect external dynamic dependencies (even within the current wb
+	 * for GNM_EXPR_REWRITE_WORKBOOK), DynDeps can not be relocated */
+	g_hash_table_foreach (hash,
+		(GHFunc) &cb_find_dynamic_deps, (gpointer) extern_dyn_dep);
+
+	/* now we invalidate things, being careful to ignore DynDeps */
+	g_hash_table_foreach (hash,
+		(GHFunc) &cb_dep_hash_invalidate, (gpointer)rwinfo);
+	g_hash_table_destroy (hash);
+}
+
 static void
 cb_name_invalidate (GnmNamedExpr *nexpr, G_GNUC_UNUSED gpointer value,
 		    GnmExprRewriteInfo const *rwinfo)
@@ -1899,6 +1903,8 @@ do_deps_destroy (Sheet *sheet, GnmExprRewriteInfo const *rwinfo)
 {
 	DependentFlags filter = DEPENDENT_LINK_FLAGS; /* unlink everything */
 	GnmDepContainer *deps;
+	GSList *ptr, *next, *local_dyn_deps, *dyn_deps = NULL;
+	Dependent *dep;
 
 	g_return_if_fail (IS_SHEET (sheet));
 
@@ -1926,11 +1932,8 @@ do_deps_destroy (Sheet *sheet, GnmExprRewriteInfo const *rwinfo)
 		int i;
 		for (i = (SHEET_MAX_ROWS-1)/BUCKET_SIZE; i >= 0 ; i--) {
 			GHashTable *hash = deps->range_hash[i];
-			if (hash != NULL) {
-				g_hash_table_foreach (hash,
-					&cb_dep_hash_invalidate, (gpointer)rwinfo);
-				g_hash_table_destroy (hash);
-			}
+			if (hash != NULL)
+				dep_hash_destroy (hash, rwinfo, &dyn_deps);
 		}
 		g_free (deps->range_hash);
 		deps->range_hash = NULL;
@@ -1941,9 +1944,7 @@ do_deps_destroy (Sheet *sheet, GnmExprRewriteInfo const *rwinfo)
 	}
 
 	if (deps->single_hash) {
-		g_hash_table_foreach (deps->single_hash,
-			&cb_dep_hash_invalidate, (gpointer)rwinfo);
-		g_hash_table_destroy (deps->single_hash);
+		dep_hash_destroy (deps->single_hash, rwinfo, &dyn_deps);
 		deps->single_hash = NULL;
 	}
 	if (deps->single_pool) {
@@ -1951,6 +1952,53 @@ do_deps_destroy (Sheet *sheet, GnmExprRewriteInfo const *rwinfo)
 		deps->single_pool = NULL;
 	}
 
+	/* Now that we have tossed all deps to this sheet we can queue the
+	 * external dyn deps for recalc and free them */
+	ptr = dyn_deps;
+	dyn_deps = local_dyn_deps = NULL;
+	for (; ptr != NULL ; ptr = next) {
+		dep = ptr->data;
+		next = ptr->next;
+		if (dep->sheet != sheet) {
+			if (dep->flags & DEPENDENT_HAS_DYNAMIC_DEPS) {
+				dependent_clear_dynamic_deps (dep);
+				dep->flags &= ~DEPENDENT_HAS_DYNAMIC_DEPS;
+			}
+			ptr->next = dyn_deps;
+			dyn_deps = ptr;
+		} else {
+			ptr->next = local_dyn_deps;
+			local_dyn_deps = ptr;
+		}
+	}
+	g_slist_free (local_dyn_deps);
+
+	/* Filter workbook local deps for GNM_EXPR_REWRITE_WORKBOOK */
+	if (rwinfo->type == GNM_EXPR_REWRITE_WORKBOOK) {
+		Workbook const *target = rwinfo->u.workbook;
+		ptr = dyn_deps;
+		dyn_deps = local_dyn_deps = NULL;
+		for (; ptr != NULL ; ptr = next) {
+			dep = ptr->data;
+			next = ptr->next;
+			if (dep->sheet->workbook != target) {
+				ptr->next = dyn_deps;
+				dyn_deps = ptr;
+			} else {
+				ptr->next = local_dyn_deps;
+				local_dyn_deps = ptr;
+			}
+		}
+	}
+	dependent_queue_recalc_list (dyn_deps);
+	g_slist_free (dyn_deps);
+	g_slist_free (local_dyn_deps);
+
+	if (deps->dynamic_deps) {
+		g_hash_table_destroy (deps->dynamic_deps);
+		deps->dynamic_deps = NULL;
+	}
+
 	if (deps->referencing_names) {
 		GSList *ptr, *accum = NULL;
 		Dependent *dep;
@@ -1981,11 +2029,6 @@ do_deps_destroy (Sheet *sheet, GnmExprRewriteInfo const *rwinfo)
 		g_hash_table_destroy (names);
 	}
 
-	if (deps->dynamic_deps) {
-		g_hash_table_destroy (deps->dynamic_deps);
-		deps->dynamic_deps = NULL;
-	}
-
 	/* TODO : when we support inter-app depends we'll need a new flag */
 	/* TODO : Add an 'application quit flag' to ignore interbook too */
 	if (sheet->deps == NULL) {
@@ -2086,12 +2129,42 @@ workbook_recalc_all (Workbook *wb)
 		sheet_update (wb_view_cur_sheet (view)););
 }
 
+static void
+dynamic_dep_free (DynamicDep *dyn)
+{
+	Dependent *dep = dyn->container;
+	CellPos const *pos = (dependent_is_cell (dep))
+		? &DEP_TO_CELL(dep)->pos : &dummy;
+	ValueRange *v;
+	GSList *ptr;
+
+	for (ptr = dyn->singles ; ptr != NULL ; ptr = ptr->next) {
+		v = ptr->data;
+		unlink_single_dep (&dyn->base, pos, &v->cell.a);
+		value_release ((Value *)v);
+	}
+	g_slist_free (dyn->singles);
+	dyn->singles = NULL;
+
+	for (ptr = dyn->ranges ; ptr != NULL ; ptr = ptr->next) {
+		v = ptr->data;
+		unlink_cellrange_dep (&dyn->base, pos, &v->cell.a, &v->cell.b);
+		value_release ((Value *)v);
+	}
+	g_slist_free (dyn->ranges);
+	dyn->ranges = NULL;
+
+	if (dyn->base.flags & DEPENDENT_HAS_3D)
+		workbook_unlink_3d_dep (&dyn->base);
+	g_free (dyn);
+}
+
 GnmDepContainer *
 gnm_dep_container_new (void)
 {
 	GnmDepContainer *deps = g_new (GnmDepContainer, 1);
 
-	deps->dependent_list = NULL;
+	deps->head = deps->tail = NULL;
 
 	deps->range_hash  = g_new0 (GHashTable *,
 				    (SHEET_MAX_ROWS-1)/BUCKET_SIZE + 1);
