@@ -25,6 +25,7 @@
 #include <gnumeric-i18n.h>
 #include <gnumeric.h>
 #include "dialogs.h"
+#include "help.h"
 
 #include <workbook-view.h>
 #include <gui-util.h>
@@ -287,7 +288,7 @@ dialog_destroy (GtkObject *w, SortFlowState  *state)
 		state->gui = NULL;
 	}
 
-	wbcg_edit_finish (state->wbcg, FALSE);
+	wbcg_edit_finish (state->wbcg, FALSE, NULL);
 
 	state->dialog = NULL;
 
@@ -598,8 +599,8 @@ cb_add_clicked (G_GNUC_UNUSED GtkWidget *w, SortFlowState *state)
 
 static void
 cb_toggled_descending (G_GNUC_UNUSED GtkCellRendererToggle *cell,
-	 gchar                 *path_string,
-	 gpointer               data)
+		       gchar                 *path_string,
+		       gpointer               data)
 {
 	SortFlowState *state = data;
 	GtkTreeModel *model = GTK_TREE_MODEL (state->model);
@@ -639,13 +640,6 @@ cb_toggled_case_sensitive (GtkCellRendererToggle *cell,
 	toggled (cell, path_string, data, ITEM_CASE_SENSITIVE);
 }
 
-/**
- * dialog_init:
- * @state:
- *
- * Create the dialog (guru).
- *
- **/
 static gboolean
 dialog_init (SortFlowState *state)
 {
@@ -797,7 +791,7 @@ dialog_init (SortFlowState *state)
 		0., .5, 0., 0.);
 	gnumeric_init_help_button (
 		glade_xml_get_widget (state->gui, "help_button"),
-		"cell-sort.html");
+		GNUMERIC_HELP_LINK_CELL_SORT);
 
 	state->ok_button  = glade_xml_get_widget (state->gui, "ok_button");
 	g_signal_connect (G_OBJECT (state->ok_button),
