@@ -25,11 +25,13 @@
 #include <gnumeric-i18n.h>
 #include <gnumeric.h>
 #include "dialogs.h"
+#include "help.h"
 
 #include <gui-util.h>
 #include <cell.h>
 #include <sheet.h>
 #include <expr.h>
+#include <commands.h>
 #include <dependent.h>
 #include <format.h>
 #include <value.h>
@@ -44,6 +46,7 @@
 #include <goal-seek.h>
 #include <mathfunc.h>
 #include <widgets/gnumeric-expr-entry.h>
+#include <selection.h>
 
 #include <math.h>
 
@@ -77,6 +80,7 @@ typedef struct {
 	Cell *old_cell;
 	Value *old_value;
 	GtkWidget *warning_dialog;
+	gboolean cancelled;
 } GoalSeekState;
 
 
@@ -259,6 +263,14 @@ dialog_destroy (GtkObject *w, GoalSeekState  *state)
 	g_return_val_if_fail (w != NULL, FALSE);
 	g_return_val_if_fail (state != NULL, FALSE);
 
+	if (!state->cancelled 
+	    && state->old_value != NULL 
+	    && state->old_cell != NULL) {
+		cmd_goal_seek (WORKBOOK_CONTROL(state->wbcg),
+			       state->old_cell, state->old_value, NULL);
+		state->old_value = NULL;
+	}
+
 	wbcg_edit_detach_guru (state->wbcg);
 
 	if (state->old_value != NULL) {
@@ -291,6 +303,8 @@ static void
 cb_dialog_cancel_clicked (G_GNUC_UNUSED GtkWidget *button,
 			  GoalSeekState *state)
 {
+	state->cancelled = TRUE;
+
 	if ((state->old_cell != NULL) && (state->old_value != NULL)) {
 		sheet_cell_set_value (state->old_cell, state->old_value);
 		workbook_recalc (state->old_cell->base.sheet->workbook);
@@ -469,6 +483,8 @@ cb_dialog_apply_clicked (G_GNUC_UNUSED GtkWidget *button,
 		gtk_label_set_text (GTK_LABEL (state->target_value_label), "");
 		break;
 	}
+	state->cancelled = FALSE;
+	
 	gtk_widget_show (state->result_table);
 	return;
 }
@@ -488,6 +504,26 @@ dialog_realized (G_GNUC_UNUSED GtkWidget *dialog,
 	gtk_widget_hide (state->result_table);
 }
 
+/**
+ * dialog_preload_selection:
+ * @state:
+ * @entry
+ *
+ *
+ **/
+static void
+dialog_preload_selection (GoalSeekState *state, GnmExprEntry *entry)
+{
+	Range const *sel;
+
+	sel = selection_first_range 
+		(wb_control_cur_sheet_view 
+		 (WORKBOOK_CONTROL (state->wbcg)), NULL, NULL);
+	if (sel)
+		gnm_expr_entry_load_from_range (entry,
+						state->sheet, sel);
+}
+
 /**
  * dialog_init:
  * @state:
@@ -520,7 +556,7 @@ dialog_init (GoalSeekState *state)
 
 	gnumeric_init_help_button (
 		glade_xml_get_widget (state->gui, "helpbutton"),
-		"goal-seek.html");
+		GNUMERIC_HELP_LINK_GOAL_SEEK);
 
 	state->to_value_entry = glade_xml_get_widget (state->gui, "to_value_entry");
 	state->at_least_entry = glade_xml_get_widget (state->gui, "at_least-entry");
@@ -547,6 +583,7 @@ dialog_init (GoalSeekState *state)
 			  0, 0);
  	gnumeric_editable_enters (GTK_WINDOW (state->dialog),
 				  GTK_WIDGET (state->set_cell_entry));
+	dialog_preload_selection (state, state->set_cell_entry);
 	gtk_widget_show (GTK_WIDGET (state->set_cell_entry));
 
 	state->change_cell_entry = gnm_expr_entry_new (state->wbcg, TRUE);
@@ -560,6 +597,7 @@ dialog_init (GoalSeekState *state)
 			  0, 0);
 	gnumeric_editable_enters (GTK_WINDOW (state->dialog),
 				  GTK_WIDGET (state->change_cell_entry));
+	dialog_preload_selection (state, state->change_cell_entry);
 	gtk_widget_show (GTK_WIDGET (state->change_cell_entry));
 
 
@@ -609,6 +647,7 @@ dialog_goal_seek (WorkbookControlGUI *wbcg, Sheet *sheet)
 	state->sheet = sheet;
 	state->gui   = gui;
 	state->warning_dialog = NULL;
+	state->cancelled = TRUE;
 
 	if (dialog_init (state)) {
 		gnumeric_notice (wbcg, GTK_MESSAGE_ERROR,
