@@ -28,6 +28,7 @@
 #include <gnumeric-i18n.h>
 #include <gnumeric.h>
 #include "dialogs.h"
+#include "help.h"
 
 #include <gui-util.h>
 #include <func.h>
@@ -40,6 +41,7 @@
 #include <workbook.h>
 #include <parse-util.h>
 #include <ranges.h>
+#include <commands.h>
 #include <widgets/gnumeric-expr-entry.h>
 
 #include <glade/glade.h>
@@ -75,6 +77,8 @@ typedef struct {
 	GSList              *ov_cell_stack;
 	GtkWidget           *warning_dialog;
 
+	gboolean             cancelled;
+
 	Sheet	            *sheet;
 	Workbook            *wb;
 	WorkbookControlGUI  *wbcg;
@@ -458,6 +462,15 @@ dialog_destroy (GtkObject *w, SolverState  *state)
 	g_return_val_if_fail (w != NULL, FALSE);
 	g_return_val_if_fail (state != NULL, FALSE);
 
+	if (state->ov_cell_stack != NULL &&
+	    !state->cancelled &&
+	    !cmd_solver (WORKBOOK_CONTROL(state->wbcg), state->ov_cell_stack, 
+			 state->ov_stack, NULL))
+	{
+		state->ov_cell_stack = NULL;
+		state->ov_stack = NULL;
+	}
+
 	if (state->ov_stack != NULL) {
 		g_slist_foreach (state->ov_stack, (GFunc)free_original_values,
 				 NULL);
@@ -530,6 +543,7 @@ cb_dialog_cancel_clicked (G_GNUC_UNUSED GtkWidget *button, SolverState *state)
 		state->ov_stack = NULL;
 		workbook_recalc (state->sheet->workbook);
 	}
+	state->cancelled = TRUE;
 
 	gtk_widget_destroy (state->dialog);
 }
@@ -545,6 +559,7 @@ static void
 cb_dialog_close_clicked (G_GNUC_UNUSED GtkWidget *button,
 			 SolverState *state)
 {
+	state->cancelled = FALSE;
 	gtk_widget_destroy (state->dialog);
 }
 
@@ -744,10 +759,10 @@ solver_reporting (SolverState *state, SolverResults *res, const gchar *errmsg)
 				((GtkWindow *) state->dialog,
 				 &(state->warning_dialog),
 				 GTK_MESSAGE_INFO,
-				 _("Sensitivity nor limits report are "
-				   "not meaningful if the program has "
-				   "integer constraints. These reports "
-				   "will thus not be created."));
+				 _("Neither sensitivity nor limits report are "
+				   "meaningful if the program has "
+				   "integer constraints.  These reports "
+				   "will not be created."));
 		err = solver_reports (WORKBOOK_CONTROL(state->wbcg),
 				      state->sheet, res,
 				      opt->answer_report,
@@ -1021,6 +1036,7 @@ cb_dialog_solve_clicked (G_GNUC_UNUSED GtkWidget *button,
 	workbook_recalc (state->sheet->workbook);
 
 	if (res != NULL) {
+		state->cancelled = FALSE;
 		solver_reporting (state, res, errmsg);
 		if (param->options.add_scenario)
 			solver_add_scenario (state, res,
@@ -1104,7 +1120,7 @@ dialog_init (SolverState *state)
 
 	gnumeric_init_help_button (glade_xml_get_widget (state->gui,
 							 "helpbutton"),
-				   "solver.html");
+				   GNUMERIC_HELP_LINK_SOLVER);
 
 	state->add_button  = glade_xml_get_widget (state->gui, "addbutton");
 	g_signal_connect (G_OBJECT (state->add_button), "clicked",
@@ -1348,6 +1364,7 @@ dialog_solver (WorkbookControlGUI *wbcg, Sheet *sheet)
 	state->ov_stack       = NULL;
 	state->ov_cell_stack  = NULL;
 	state->warning_dialog = NULL;
+	state->cancelled      = TRUE;
 
 	if (dialog_init (state)) {
 		gnumeric_notice (wbcg, GTK_MESSAGE_ERROR,
