@@ -5,6 +5,9 @@
  * Author:
  *  Miguel de Icaza (miguel@gnu.org)
  */
+
+#undef GTK_DISABLE_DEPRECATED
+#warning "This file uses GTK_DISABLE_DEPRECATED"
 #include <gnumeric-config.h>
 #include <gnumeric-i18n.h>
 #include "gnumeric.h"
@@ -21,9 +24,11 @@
 #include "format.h"
 #include "application.h"
 #include "workbook.h"
+#include "gnumeric-paths.h"
 
 #include <string.h>
-#include <gal/widgets/widget-color-combo.h>
+#include <widgets/widget-color-combo.h>
+#include <widgets/preview-file-selection.h>
 #include <glade/glade.h>
 
 #ifdef WITH_BONOBO
@@ -32,6 +37,8 @@
 
 #include <gdk/gdkkeysyms.h>
 #include <libgnome/gnome-help.h>
+#include <libgnomeui/gnome-app.h>
+#include <libgnomeui/gnome-app-helper.h>
 
 gboolean
 gnumeric_dialog_question_yes_no (WorkbookControlGUI *wbcg,
@@ -165,10 +172,13 @@ fsel_key_event (GtkWidget *widget, GdkEventKey *event, gpointer user_data)
 gboolean
 gnumeric_dialog_file_selection (WorkbookControlGUI *wbcg, GtkFileSelection *fsel)
 {
+	/* Note: wbcg will be NULL if called (indirectly) from gog-style.c  */
 	gboolean result = FALSE;
 
 	gtk_window_set_modal (GTK_WINDOW (fsel), TRUE);
-	gnumeric_set_transient (wbcg, GTK_WINDOW (fsel));
+	if (wbcg)
+		gnumeric_set_transient (wbcg_toplevel (wbcg), 
+					GTK_WINDOW (fsel));
 	g_signal_connect (G_OBJECT (fsel->ok_button),
 		"clicked",
 		G_CALLBACK (fsel_handle_ok), &result);
@@ -194,7 +204,7 @@ gnumeric_dialog_dir_selection (WorkbookControlGUI *wbcg, GtkFileSelection *fsel)
 	gboolean result = FALSE;
 
 	gtk_window_set_modal (GTK_WINDOW (fsel), TRUE);
-	gnumeric_set_transient (wbcg, GTK_WINDOW (fsel));
+	gnumeric_set_transient (wbcg_toplevel (wbcg), GTK_WINDOW (fsel));
 	g_signal_connect (G_OBJECT (fsel->ok_button),
 		"clicked",
 		G_CALLBACK (fsel_dir_handle_ok), &result);
@@ -214,6 +224,17 @@ gnumeric_dialog_dir_selection (WorkbookControlGUI *wbcg, GtkFileSelection *fsel)
 	return result;
 }
 
+static gint
+cb_modal_dialog_keypress (GtkWidget *w, GdkEventKey *e)
+{
+	if(e->keyval == GDK_Escape) {
+		gtk_dialog_response (GTK_DIALOG (w), GTK_RESPONSE_CANCEL);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 /**
  * gnumeric_dialog_run
  *
@@ -232,6 +253,10 @@ gnumeric_dialog_run (WorkbookControlGUI *wbcg, GtkDialog *dialog)
 		wbcg_set_transient (wbcg, GTK_WINDOW (dialog));
 	}
 
+	g_signal_connect (G_OBJECT (dialog),
+		"key-press-event",
+		G_CALLBACK (cb_modal_dialog_keypress), NULL);
+
 	result = gtk_dialog_run (dialog);
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 	return result;
@@ -246,11 +271,11 @@ insert_error_info (GtkTextBuffer* text, ErrorInfo *error, gint level)
 	gchar *message = (gchar *) error_info_peek_message (error);
 	GSList *details_list, *l;
 	GtkTextIter start, last;
-	gchar *tag_name = g_strdup_printf (ERROR_INFO_TAG_NAME, 
-					   MIN (level, ERROR_INFO_MAX_LEVEL)); 
+	gchar *tag_name = g_strdup_printf (ERROR_INFO_TAG_NAME,
+					   MIN (level, ERROR_INFO_MAX_LEVEL));
 	if (message == NULL)
 		message = g_strdup (_("Multiple errors\n"));
-	else 
+	else
 		message = g_strdup_printf ("%s\n", message);
 	gtk_text_buffer_get_bounds (text, &start, &last);
 	gtk_text_buffer_insert_with_tags_by_name (text, &last,
@@ -281,6 +306,7 @@ gnumeric_error_info_dialog_new (ErrorInfo *error)
 	gchar *message;
 	gint bf_lim = 1;
 	gint i;
+	GdkScreen *screen;
 
 	g_return_val_if_fail (error != NULL, NULL);
 
@@ -293,9 +319,12 @@ gnumeric_error_info_dialog_new (ErrorInfo *error)
 		mtype = GTK_MESSAGE_WARNING;
 	dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
 					 mtype, GTK_BUTTONS_CLOSE, " ");
-	gtk_widget_set_usize (dialog, 450, 250);
+	screen = gtk_widget_get_screen (dialog);
+	gtk_widget_set_size_request (dialog,
+				     gdk_screen_get_width (screen) / 3,
+				     gdk_screen_get_width (screen) / 4);
 	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), 
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_shadow_type
@@ -326,6 +355,7 @@ gnumeric_error_info_dialog_new (ErrorInfo *error)
 	gtk_widget_show_all (GTK_WIDGET (scrolled_window));
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), scrolled_window, TRUE, TRUE, 0);
 
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
 	return dialog;
 }
 
@@ -337,7 +367,6 @@ void
 gnumeric_error_info_dialog_show (WorkbookControlGUI *wbcg, ErrorInfo *error)
 {
 	GtkWidget *dialog = gnumeric_error_info_dialog_new (error);
-	gnumeric_set_transient (wbcg, GTK_WINDOW (dialog));
 	gnumeric_dialog_run (wbcg, GTK_DIALOG (dialog));
 }
 
@@ -364,22 +393,20 @@ cb_parent_mapped (GtkWidget *parent, GtkWindow *window)
  * a GnomeDialog.
  */
 void
-gnumeric_set_transient (WorkbookControlGUI *wbcg, GtkWindow *window)
+gnumeric_set_transient (GtkWindow *toplevel, GtkWindow *window)
 {
-	GtkWindow *toplevel;
 /* FIXME:                                                                     */
 /* 	GtkWindowPosition position = gnome_preferences_get_dialog_position(); */
 	GtkWindowPosition position = GTK_WIN_POS_CENTER_ON_PARENT;
 
-	g_return_if_fail (IS_WORKBOOK_CONTROL_GUI (wbcg));
+	g_return_if_fail (GTK_IS_WINDOW (toplevel));
 	g_return_if_fail (GTK_IS_WINDOW (window));
 
-	toplevel = wbcg_toplevel (wbcg);
 	gtk_window_set_transient_for (window, toplevel);
 
 	if (position == GTK_WIN_POS_NONE)
 		position = GTK_WIN_POS_CENTER_ON_PARENT;
-	gtk_window_set_position(window, position);
+	gtk_window_set_position (window, position);
 
 	if (!GTK_WIDGET_MAPPED (toplevel))
 		g_signal_connect_after (G_OBJECT (toplevel),
@@ -402,11 +429,26 @@ cb_free_keyed_dialog_context (KeyedDialogContext *ctxt)
 	ctxt->freed = TRUE;
 
 	g_return_if_fail (g_object_get_data (G_OBJECT (ctxt->wbcg), ctxt->key));
+	/*
+	 * One of these causes a recursive call which will do nothing due to
+	 * ->freed.
+	 */
 	g_object_set_data (G_OBJECT (ctxt->wbcg), ctxt->key, NULL);
-	g_object_set_data (G_OBJECT (ctxt->wbcg), "KeyedDialog", NULL);
+	g_object_set_data (G_OBJECT (ctxt->dialog), "KeyedDialog", NULL);
 	g_free (ctxt);
 }
 
+static gint
+cb_keyed_dialog_keypress (GtkWidget *dialog, GdkEventKey *event,
+			  G_GNUC_UNUSED gpointer user)
+{
+	if (event->keyval == GDK_Escape) {
+		gtk_object_destroy (GTK_OBJECT (dialog));
+		return TRUE;
+	}
+	return FALSE;
+}
+
 /**
  * gnumeric_keyed_dialog
  *
@@ -439,6 +481,9 @@ gnumeric_keyed_dialog (WorkbookControlGUI *wbcg, GtkWindow *dialog, const char *
 		key, ctxt, (GDestroyNotify) cb_free_keyed_dialog_context);
 	g_object_set_data_full (G_OBJECT (dialog),
 		"KeyedDialog", ctxt, (GDestroyNotify) cb_free_keyed_dialog_context);
+	g_signal_connect (G_OBJECT (dialog),
+		"key_press_event",
+		G_CALLBACK (cb_keyed_dialog_keypress), NULL);
 }
 
 /**
@@ -489,8 +534,8 @@ gnumeric_editable_enters (GtkWindow *window, GtkWidget *w)
 	g_return_if_fail (GTK_IS_WINDOW(window));
 
 	/* because I really do not feel like changing all the calls to this routine */
-	if (IS_GNUMERIC_EXPR_ENTRY (w))
-		w = GTK_WIDGET (gnm_expr_entry_get_entry (GNUMERIC_EXPR_ENTRY (w)));
+	if (IS_GNM_EXPR_ENTRY (w))
+		w = GTK_WIDGET (gnm_expr_entry_get_entry (GNM_EXPR_ENTRY (w)));
 
 	g_signal_connect_swapped (G_OBJECT (w),
 		"activate",
@@ -650,7 +695,7 @@ gnumeric_popup_menu (GtkMenu *menu, GdkEventButton *event)
 	g_return_if_fail (menu != NULL);
 	g_return_if_fail (GTK_IS_MENU (menu));
 
-	gtk_object_ref (GTK_OBJECT (menu));
+	g_object_ref (menu);
 	gtk_object_sink (GTK_OBJECT (menu));
 
 	g_signal_connect (G_OBJECT (menu),
@@ -664,49 +709,6 @@ gnumeric_popup_menu (GtkMenu *menu, GdkEventButton *event)
 	gtk_menu_popup (menu, NULL, NULL, 0, NULL, 0, event->time);
 }
 
-/*
- * Helper for gnumeric_clist_moveto. Ensures that we move in the same way
- * whether direct or from a callback.
- */
-static void
-clist_moveto (GtkCList *clist, gint row)
-{
-	gtk_clist_moveto (clist, row, 0, 0.1, 0.0);
-	clist->focus_row = row;
-}
-
-/*
- * map handler. Disconnects itself and moves the list.
- */
-static void
-cb_clist_moveto (GtkWidget *clist, gpointer row)
-{
-	g_signal_handlers_disconnect_by_func (G_OBJECT (clist),
-		G_CALLBACK (cb_clist_moveto), row);
-
-	clist_moveto (GTK_CLIST (clist), GPOINTER_TO_INT (row));
-}
-
-/*
- * gnumeric_clist_moveto
- * @clist   clist
- * @row     row
-
- *
- * Scroll the viewing area of the list to the given row.
- * We do it this way because gtk_clist_moveto only works if the list is
- * mapped.
- */
-void
-gnumeric_clist_moveto (GtkCList *clist, gint row)
-{
-	if (GTK_WIDGET_DRAWABLE (clist))
-		clist_moveto (clist, row);
-	else
-		g_signal_connect (G_OBJECT (clist),
-			"map",
-			G_CALLBACK (cb_clist_moveto), GINT_TO_POINTER (row));
-}
 
 GtkWidget *
 gnumeric_create_tooltip (void)
@@ -746,37 +748,53 @@ gnumeric_position_tooltip (GtkWidget *tip, int horizontal)
 
 	gtk_widget_size_request (tip, &req);
 	gdk_window_get_pointer (NULL, &x, &y, NULL);
+
 	if (horizontal){
-		x = x - req.width/2;
-		y = y - req.height - 20;
+		x -= req.width / 2;
+		y -= req.height + 20;
 	} else {
-		x = x - req.width - 20;
-		y = y - req.height/2;
+		x -= req.width + 20;
+		y -= req.height / 2;
 	}
 
 	if (x < 0)
 		x = 0;
 	if (y < 0)
 		y = 0;
-	gtk_widget_set_uposition (gtk_widget_get_toplevel (tip), x, y);
+
+	gtk_window_move (GTK_WINDOW (gtk_widget_get_toplevel (tip)), x, y);
 }
 
+/**
+ * gnm_glade_xml_new :
+ * @cc : #CommandContext
+ * @gladefile :
+ *
+ * Simple utility to open glade files
+ **/
 GladeXML *
-gnumeric_glade_xml_new (WorkbookControlGUI *wbcg, char const * gladefile)
+gnm_glade_xml_new (CommandContext *cc, char const *gladefile,
+		   char const *root, char const *domain)
 {
 	GladeXML *gui;
-	char *d = gnumeric_sys_glade_dir ();
-	char *f = g_build_filename (d, gladefile, NULL);
-	gui = glade_xml_new (f, NULL, "gnumeric");
+	char *f;
 
-	/* Onlt report errors if the context is non-null */
-	if (gui == NULL && wbcg != NULL) {
+	g_return_val_if_fail (gladefile != NULL, NULL);
+
+	if (!g_path_is_absolute (gladefile)) {
+		char *d = gnumeric_sys_glade_dir ();
+		f = g_build_filename (d, gladefile, NULL);
+		g_free (d);
+	} else
+		f = g_strdup (gladefile);
+
+	gui = glade_xml_new (f, root, domain);
+	if (gui == NULL && cc != NULL) {
 		char *msg = g_strdup_printf (_("Unable to open file '%s'"), f);
-		gnumeric_error_system (COMMAND_CONTEXT (wbcg), msg);
+		gnumeric_error_system (cc, msg);
 		g_free (msg);
 	}
 	g_free (f);
-	g_free (d);
 
 	return gui;
 }
@@ -793,10 +811,9 @@ cb_non_modal_dialog_keypress (GtkWidget *w, GdkEventKey *e)
 }
 
 void
-gnumeric_non_modal_dialog (WorkbookControlGUI *wbcg, GtkWindow *dialog)
+gnumeric_non_modal_dialog (GtkWindow *toplevel, GtkWindow *dialog)
 {
-	gnumeric_set_transient (wbcg, dialog);
-
+	gnumeric_set_transient (toplevel, dialog);
 	g_signal_connect (G_OBJECT (dialog),
 		"key-press-event",
 		G_CALLBACK (cb_non_modal_dialog_keypress), NULL);
@@ -812,14 +829,21 @@ void
 gnumeric_inject_widget_into_bonoboui (WorkbookControlGUI *wbcg, GtkWidget *widget, char const *path)
 {
 	BonoboControl *control;
+	CORBA_Environment ev;
 
 	gtk_widget_show_all (widget);
 	control = bonobo_control_new (widget);
+
+	CORBA_exception_init (&ev);
 	bonobo_ui_component_object_set (
 		wbcg->uic, path,
 		BONOBO_OBJREF (control),
-		NULL);
-	bonobo_object_unref (BONOBO_OBJECT (control));
+		&ev);
+	/* if there was a problem injecting the widget then nothing is refing
+	 * the control and the widget will go away under our feet */
+	if (!BONOBO_EX (&ev))
+		bonobo_object_unref (BONOBO_OBJECT (control));
+	CORBA_exception_free (&ev);
 }
 #endif
 
@@ -827,9 +851,9 @@ static void
 popup_item_activate (GtkWidget *item, gpointer *user_data)
 {
 	GnumericPopupMenuElement const *elem =
-		gtk_object_get_data (GTK_OBJECT (item), "descriptor");
+		g_object_get_data (G_OBJECT (item), "descriptor");
 	GnumericPopupMenuHandler handler =
-		gtk_object_get_data (GTK_OBJECT (item), "handler");
+		g_object_get_data (G_OBJECT (item), "handler");
 
 	g_return_if_fail (elem != NULL);
 	g_return_if_fail (handler != NULL);
@@ -886,14 +910,14 @@ gnumeric_create_popup_menu_list (GSList *elements,
 			g_signal_connect (G_OBJECT (item),
 				"activate",
 				G_CALLBACK (&popup_item_activate), user_data);
-			gtk_object_set_data (
-				GTK_OBJECT (item), "descriptor", (gpointer)(element));
-			gtk_object_set_data (
-				GTK_OBJECT (item), "handler", (gpointer)handler);
+			g_object_set_data (
+				G_OBJECT (item), "descriptor", (gpointer)(element));
+			g_object_set_data (
+				G_OBJECT (item), "handler", (gpointer)handler);
 		}
 
 		gtk_widget_show (item);
-		gtk_menu_append (GTK_MENU (menu), item);
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 	}
 
 	gnumeric_popup_menu (GTK_MENU (menu), event);
@@ -926,14 +950,18 @@ gnumeric_create_popup_menu (GnumericPopupMenuElement const *elements,
 StyleColor *
 color_combo_get_style_color (GtkWidget *color_combo)
 {
+	StyleColor *sc = NULL;
 	GdkColor *gdk = color_combo_get_color (COLOR_COMBO (color_combo), NULL);
-	return (gdk == NULL) ? NULL
-		    : style_color_new (gdk->red, gdk->green, gdk->blue);
+	if (gdk != NULL) {
+		    sc = style_color_new (gdk->red, gdk->green, gdk->blue);
+		    gdk_color_free (gdk);
+	}
+	return sc;
 }
 
 GtkWidget *
 gnumeric_toolbar_new (WorkbookControlGUI *wbcg,
-		      GnomeUIInfo *info,
+		      gpointer info, /* GnomeUIInfo *info */
 		      char const *name,
 		      gint band_num,
 		      gint band_position,
@@ -1062,11 +1090,11 @@ focus_on_entry (GtkEntry *entry)
 		return;
 	gtk_widget_grab_focus (GTK_WIDGET(entry));
 	gtk_editable_set_position (GTK_EDITABLE (entry), 0);
-	gtk_entry_select_region (entry, 0, entry->text_length);
+	gtk_editable_select_region (GTK_EDITABLE (entry), 0, entry->text_length);
 }
 
 gboolean
-entry_to_float_with_format_default (GtkEntry *entry, gnm_float *the_float, gboolean update, 
+entry_to_float_with_format_default (GtkEntry *entry, gnm_float *the_float, gboolean update,
 				    StyleFormat *format, gnm_float num)
 {
 	char const *text = gtk_entry_get_text (entry);
@@ -1090,10 +1118,10 @@ entry_to_float_with_format_default (GtkEntry *entry, gnm_float *the_float, gbool
 }
 
 gboolean
-entry_to_float_with_format (GtkEntry *entry, gnm_float *the_float, gboolean update, 
+entry_to_float_with_format (GtkEntry *entry, gnm_float *the_float, gboolean update,
 			    StyleFormat *format)
 {
-	Value *value = format_match_number (gtk_entry_get_text (entry), format);
+	Value *value = format_match_number (gtk_entry_get_text (entry), format, NULL);
 
 	if ((value == NULL) || !VALUE_IS_NUMBER (value)) {
 		*the_float = 0.0;
@@ -1101,7 +1129,7 @@ entry_to_float_with_format (GtkEntry *entry, gnm_float *the_float, gboolean upda
 	}
 	*the_float = value_get_as_float (value);
 	if (update) {
-		char *tmp = format_value (format, value, NULL, 16);
+		char *tmp = format_value (format, value, NULL, 16, NULL);
 		gtk_entry_set_text (entry, tmp);
 		g_free (tmp);
 	}
@@ -1122,7 +1150,7 @@ entry_to_float_with_format (GtkEntry *entry, gnm_float *the_float, gboolean upda
 gboolean
 entry_to_int (GtkEntry *entry, gint *the_int, gboolean update)
 {
-	Value *value = format_match_number (gtk_entry_get_text (entry), NULL);
+	Value *value = format_match_number (gtk_entry_get_text (entry), NULL, NULL);
 
 	if ((value == NULL) || !(value->type == VALUE_INTEGER)) {
 		*the_int = 0;
@@ -1130,7 +1158,7 @@ entry_to_int (GtkEntry *entry, gint *the_int, gboolean update)
 	}
 	*the_int = value_get_as_int (value);
 	if (update) {
-		char *tmp = format_value (NULL, value, NULL, 16);
+		char *tmp = format_value (NULL, value, NULL, 16, NULL);
 		gtk_entry_set_text (entry, tmp);
 		g_free (tmp);
 	}
@@ -1148,16 +1176,13 @@ entry_to_int (GtkEntry *entry, gint *the_int, gboolean update)
 void
 float_to_entry (GtkEntry *entry, gnm_float the_float)
 {
-	char        *text      = NULL;
-	Value       *val = NULL;
-
-	val = value_new_float (the_float);
-	text = format_value (NULL, val, NULL, 16);
-	if (text) {
+	Value *val  = value_new_float (the_float);
+	char  *text = format_value (NULL, val, NULL, 16, NULL);
+	if (text != NULL) {
 		gtk_entry_set_text (entry, text);
 		g_free (text);
 	}
-	if (val)
+	if (val != NULL)
 		value_release(val);
 }
 
@@ -1171,28 +1196,21 @@ float_to_entry (GtkEntry *entry, gnm_float the_float)
 void
 int_to_entry (GtkEntry *entry, gint the_int)
 {
-	char        *text      = NULL;
-	Value       *val = NULL;
-
-	val = value_new_int (the_int);
-	text = format_value (NULL, val, NULL, 16);
-	if (text) {
+	Value *val  = value_new_int (the_int);
+	char  *text = format_value (NULL, val, NULL, 16, NULL);
+	if (text != NULL) {
 		gtk_entry_set_text (entry, text);
 		g_free (text);
 	}
-	if (val)
+	if (val != NULL)
 		value_release(val);
-	return;
 }
 
 GtkWidget *
 gnumeric_load_image (char const *filename)
 {
-	GtkWidget *image;
-	char *path;
-
-	path = g_strconcat (GNUMERIC_ICONDIR "/", filename, NULL);
-	image = gtk_image_new_from_file (path);
+	char *path = g_strconcat (GNUMERIC_ICONDIR "/", filename, NULL);
+	GtkWidget *image = gtk_image_new_from_file (path);
 	g_free (path);
 
 	if (image)
@@ -1208,11 +1226,486 @@ gnumeric_load_image (char const *filename)
 GdkPixbuf *
 gnumeric_load_pixbuf (char const *filename)
 {
-	GdkPixbuf *pixbuf;
-	char *path;
-
-	path = g_strconcat (GNUMERIC_ICONDIR "/", filename, NULL);
-	pixbuf = gdk_pixbuf_new_from_file (path, NULL);
+	char *path = g_strconcat (GNUMERIC_ICONDIR "/", filename, NULL);
+	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file (path, NULL);
 	g_free (path);
 	return pixbuf;
 }
+
+
+/**
+ * gnm_pixbuf_tile: created a pixbuf consistent of the source pixbuf tiled
+ * enough times to fill out the space needed.
+ *
+ * The fractional tiles are spead evenly left-right and top-bottom.
+ */
+GdkPixbuf *
+gnm_pixbuf_tile (const GdkPixbuf *src, int w, int h)
+{
+	int src_w = gdk_pixbuf_get_width (src);
+	int src_h = gdk_pixbuf_get_height (src);
+
+	int tile_x = w / src_w;  /* Number of full tiles  */
+	int tile_y = h / src_h;
+
+	int left_x = w - tile_x * src_w;
+	int left_y = h - tile_y * src_h;
+
+	int dst_y = 0;
+	int stripe_y;
+	GdkPixbuf *dst = gdk_pixbuf_new (gdk_pixbuf_get_colorspace (src),
+					 gdk_pixbuf_get_has_alpha (src),
+					 gdk_pixbuf_get_bits_per_sample (src),
+					 w, h);
+
+	for (stripe_y = -1; stripe_y <= tile_y; stripe_y++) {
+		int dst_x = 0;
+		int stripe_x;
+		int this_h, start_y = 0;
+
+		if (stripe_y == -1) {
+			this_h = (left_y + 1) / 2;
+			start_y = src_h - this_h;
+		} else if (stripe_y == tile_y)
+			this_h = left_y / 2;
+		else
+			this_h = src_h;
+
+		if (this_h == 0)
+			continue;
+
+		for (stripe_x = -1; stripe_x <= tile_x; stripe_x++) {
+			int this_w, start_x = 0;
+
+			if (stripe_x == -1) {
+				this_w = (left_x + 1) / 2;
+				start_x = src_w - this_w;
+			} else if (stripe_x == tile_x)
+				this_w = left_x / 2;
+			else
+				this_w = src_w;
+
+			if (this_w == 0)
+				continue;
+
+			gdk_pixbuf_copy_area (src, start_x, start_y,
+					      this_w, this_h,
+					      dst,
+					      dst_x, dst_y);
+
+			dst_x += this_w;
+		}
+
+		dst_y += this_h;
+	}
+
+	return dst;
+}
+
+
+static void
+add_atk_relation (GtkWidget *w0, GtkWidget *w1, AtkRelationType type)
+{
+	AtkObject *atk0 = gtk_widget_get_accessible(w0);
+	AtkObject *atk1 = gtk_widget_get_accessible(w1);
+	AtkRelationSet *relation_set = atk_object_ref_relation_set (atk0);
+	AtkRelation *relation = atk_relation_new (&atk1, 1, type);
+	atk_relation_set_add (relation_set, relation);
+	g_object_unref (relation_set);
+	g_object_unref (relation);
+}
+
+/**
+ * gnm_setup_label_atk :
+ * @label : #GtkLabel
+ * @target : #GtkWidget
+ *
+ * A convenience routine to setup label-for/labeled-by relationship between a
+ * pair of widgets
+ **/
+void
+gnm_setup_label_atk (GtkLabel *label, GtkWidget *target)
+{
+	 add_atk_relation (GTK_WIDGET (label), target, ATK_RELATION_LABEL_FOR);
+	 add_atk_relation (target, GTK_WIDGET (label), ATK_RELATION_LABELLED_BY);
+}
+
+
+int
+gnm_measure_string (PangoContext *context, const PangoFontDescription *font_desc, const char *str)
+{
+	PangoLayout *layout = pango_layout_new (context);
+	int width;
+
+	pango_layout_set_text (layout, str, -1);
+	pango_layout_set_font_description (layout, font_desc);
+	pango_layout_get_pixel_size (layout, &width, NULL);
+
+	g_object_unref (layout);
+
+	return width;
+}
+
+static void
+cb_focus_to_entry (GtkWidget *button, GtkWidget *entry)
+{
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)))
+		gtk_widget_grab_focus (entry);
+}
+
+static gboolean
+cb_activate_button (G_GNUC_UNUSED GtkWidget *widget,
+		    G_GNUC_UNUSED GdkEventFocus *event,
+		    GtkWidget *button)
+{
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+	return FALSE;
+}
+
+void
+gnm_link_button_and_entry (GtkWidget *button, GtkWidget *entry)
+{
+	g_signal_connect (G_OBJECT (button),
+			  "clicked", G_CALLBACK (cb_focus_to_entry),
+			  entry);
+	g_signal_connect (G_OBJECT (entry),
+			  "focus_in_event",
+			  G_CALLBACK (cb_activate_button),
+			  button);
+}
+
+/* ------------------------------------------------------------------------- */
+
+void
+gnm_widget_set_cursor (GtkWidget *w, GdkCursor *cursor)
+{
+	gdk_window_set_cursor (w->window, cursor);
+}
+
+void
+gnm_widget_set_cursor_type (GtkWidget *w, GdkCursorType ct)
+{
+	GdkDisplay *display = gdk_drawable_get_display (w->window);
+	GdkCursor *cursor = gdk_cursor_new_for_display (display, ct);
+	gnm_widget_set_cursor (w, cursor);
+	gdk_cursor_unref (cursor);
+}
+
+#ifndef HAVE_GDK_CURSOR_NEW_FROM_PIXBUF
+/* See http://bugzilla.gnome.org/showattachment.cgi?attach_id=17695 */
+static GdkCursor *
+gdk_cursor_new_from_pixbuf (GdkDisplay *display, 
+			    GdkPixbuf  *pixbuf,
+			    gint        x,
+			    gint        y)
+{
+  GdkCursor *cursor;
+  GdkPixmap *pixmap, *mask;
+  guint width, height, n_channels, rowstride, i, j;
+  guint8 *data, *mask_data, *pixels;
+  GdkColor fg = { 0, 0, 0, 0 };
+  GdkColor bg = { 0, 0xffff, 0xffff, 0xffff };
+  GdkScreen *screen;
+
+  g_return_val_if_fail (GDK_IS_DISPLAY (display), NULL);
+  g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), NULL);
+
+  width = gdk_pixbuf_get_width (pixbuf);
+  height = gdk_pixbuf_get_height (pixbuf);
+
+  g_return_val_if_fail (0 <= x && (guint)x < width, NULL);
+  g_return_val_if_fail (0 <= y && (guint)y < height, NULL);
+
+  n_channels = gdk_pixbuf_get_n_channels (pixbuf);
+  rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+  pixels = gdk_pixbuf_get_pixels (pixbuf);
+
+  data = g_new0 (guint8, (width + 7) / 8 * height);
+  mask_data = g_new0 (guint8, (width + 7) / 8 * height);
+
+  for (j = 0; j < height; j++)
+    {
+      guint8 *src = pixels + j * rowstride;
+      guint8 *d = data + (width + 7) / 8 * j;
+      guint8 *md = mask_data + (width + 7) / 8 * j;
+	
+      for (i = 0; i < width; i++)
+	{
+	  if (src[1] < 0x80)
+	    *d |= 1 << (i % 8);
+	  
+	  if (n_channels == 3 || src[3] >= 0x80) {
+	    *md |= 1 << (i % 8);
+	  }
+
+	  src += n_channels;
+	  if (i % 8 == 7)
+		  d++, md++;
+	}
+    }
+      
+  screen = gdk_display_get_default_screen (display);
+  pixmap = gdk_bitmap_create_from_data (gdk_screen_get_root_window (screen), 
+					data, width, height);
+  g_free (data);
+
+  mask = gdk_bitmap_create_from_data (gdk_screen_get_root_window (screen),
+				      mask_data, width, height);
+  g_free (mask_data);
+   
+  cursor = gdk_cursor_new_from_pixmap (pixmap, mask, &fg, &bg, x, y);
+   
+  g_object_unref (pixmap);
+  g_object_unref (mask);
+  
+  return cursor;
+}
+#endif
+
+GdkCursor *
+gnm_fat_cross_cursor (GdkDisplay *display)
+{
+	/* We don't actually own a ref, but that's ok.  */
+	static GdkPixbuf *pixbuf = NULL;
+
+	if (!pixbuf)
+		pixbuf = application_get_pixbuf ("cursor_cross");
+
+	return gdk_cursor_new_from_pixbuf (display, pixbuf, 17, 17);
+}
+
+/* ------------------------------------------------------------------------- */
+
+/**
+ * gnumeric_button_new_with_stock_image
+ *
+ * Code from gedit
+ *
+ * Creates a new GtkButton with custom label and stock image.
+ * 
+ * text : button label
+ * sotck_id : id for stock icon
+ *
+ * return : newly created button
+ *
+ **/
+
+GtkWidget* 
+gnumeric_button_new_with_stock_image (const gchar* text, const gchar* stock_id)
+{
+	GtkWidget *button;
+	GtkStockItem item;
+	GtkWidget *label;
+	GtkWidget *image;
+	GtkWidget *hbox;
+	GtkWidget *align;
+
+	button = gtk_button_new ();
+
+	if (GTK_BIN (button)->child)
+		gtk_container_remove (GTK_CONTAINER (button),
+				      GTK_BIN (button)->child);
+
+	if (gtk_stock_lookup (stock_id, &item)) {
+		label = gtk_label_new_with_mnemonic (text);
+
+		gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
+
+		image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON);
+		hbox = gtk_hbox_new (FALSE, 2);
+
+		align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+
+		gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+		gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
+		gtk_container_add (GTK_CONTAINER (button), align);
+		gtk_container_add (GTK_CONTAINER (align), hbox);
+		gtk_widget_show_all (align);
+
+		return button;
+	}
+
+	label = gtk_label_new_with_mnemonic (text);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (button));
+
+	gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0.5);
+
+	gtk_widget_show (label);
+	gtk_container_add (GTK_CONTAINER (button), label);
+
+	return button;
+}
+
+/**
+ * gnumeric_dialog_add_button
+ *
+ * Code from gedit
+ *
+ * Creates and adds a button with stock image to the action area of an existing dialog.
+ * 
+ * dialog : dialog you want to add a button
+ * text : button label
+ * sotck_id : stock icon id
+ * response_id : respond id when button clicked
+ *
+ * return : newly created button
+ *
+ **/
+
+GtkWidget*
+gnumeric_dialog_add_button (GtkDialog *dialog, const gchar* text, const gchar* stock_id,
+			    gint response_id)
+{
+	GtkWidget *button;
+
+	g_return_val_if_fail (GTK_IS_DIALOG (dialog), NULL);
+	g_return_val_if_fail (text != NULL, NULL);
+	g_return_val_if_fail (stock_id != NULL, NULL);
+
+	button = gnumeric_button_new_with_stock_image (text, stock_id);
+	g_return_val_if_fail (button != NULL, NULL);
+
+	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
+
+	gtk_widget_show (button);
+
+	gtk_dialog_add_action_widget (dialog, button, response_id);	
+
+	return button;
+}
+
+/**
+ * gnumeric_message_dialog_new :
+ *
+ * A convenience fonction to build HIG compliant message dialogs.
+ *
+ *   parent : transient parent, or NULL for none.
+ *   flags 
+ *   type : type of dialog
+ *   primary_message : message displayed in bold
+ *   secondary_message : message displayed below
+ *
+ *   return : a GtkDialog, without buttons.
+ **/
+
+GtkWidget *
+gnumeric_message_dialog_new (GtkWindow * parent,
+			     GtkDialogFlags flags,
+			     GtkMessageType type,
+			     gchar const * primary_message,
+			     gchar const * secondary_message)
+{
+	GtkWidget * dialog;
+	GtkWidget * label;
+	GtkWidget * image;
+	GtkWidget * hbox;
+	gchar * message;
+	const gchar *stock_id = NULL;
+	GtkStockItem item;
+
+	dialog = gtk_dialog_new_with_buttons ("", parent, flags, NULL);
+
+	if (dialog) {
+		image = gtk_image_new (); 
+
+		switch (type) {
+		case GTK_MESSAGE_INFO:
+			stock_id = GTK_STOCK_DIALOG_INFO;
+			break;
+
+		case GTK_MESSAGE_QUESTION:
+			stock_id = GTK_STOCK_DIALOG_QUESTION;
+			break;
+
+		case GTK_MESSAGE_WARNING:
+			stock_id = GTK_STOCK_DIALOG_WARNING;
+			break;
+
+		case GTK_MESSAGE_ERROR:
+			stock_id = GTK_STOCK_DIALOG_ERROR;
+			break;
+
+		default:
+			g_warning ("Unknown GtkMessageType %d", type);
+			break;
+		}
+
+		if (stock_id == NULL)
+			stock_id = GTK_STOCK_DIALOG_INFO;
+
+		if (gtk_stock_lookup (stock_id, &item)) {
+			gtk_image_set_from_stock (GTK_IMAGE (image), stock_id,
+						  GTK_ICON_SIZE_DIALOG);
+
+			gtk_window_set_title (GTK_WINDOW (dialog), item.label);
+		} else
+			g_warning ("Stock dialog ID doesn't exist?");  
+
+		if (primary_message) {
+			if (secondary_message) {
+				message = g_strdup_printf ("<b>%s</b>\n\n%s",
+							   primary_message,
+							   secondary_message);
+			} else {
+				message = g_strdup_printf ("<b>%s</b>",
+							   primary_message);
+			}
+		} else {
+			message = g_strdup_printf (secondary_message);
+		}
+		label = gtk_label_new (message);
+		g_free (message);
+
+		hbox = gtk_hbox_new (FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, TRUE, 0);
+		gtk_box_pack_start_defaults (GTK_BOX (hbox),
+					     label);
+		gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+					     hbox);
+
+		gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+		gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+		gtk_misc_set_alignment (GTK_MISC (label), 0.0 , 0.0);
+		gtk_misc_set_alignment (GTK_MISC (label), 0.0 , 0.0);
+		gtk_box_set_spacing (GTK_BOX (hbox), 12);
+		gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
+		gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 12);
+		gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
+		gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+		gtk_window_set_resizable (GTK_WINDOW(dialog), FALSE);
+		gtk_widget_show_all (GTK_WIDGET (GTK_DIALOG (dialog)->vbox));
+	}
+
+	return dialog;
+}
+
+GdkPixbuf*
+gnm_pixbuf_intelligent_scale (GdkPixbuf *buf, guint width, guint height)
+{
+	GdkPixbuf *scaled;
+	int w, h;
+	unsigned long int ow = gdk_pixbuf_get_width (buf);
+	unsigned long int oh = gdk_pixbuf_get_height (buf);
+
+	if (ow <= width && oh <= height)
+		scaled = g_object_ref (buf);
+	else
+	{
+		if (ow * height > oh * width)
+		{
+			w = width;
+			h = width * (((double)oh)/(double)ow);
+		}
+		else
+		{
+			h = height;
+			w = height * (((double)ow)/(double)oh);
+		}
+			
+		scaled = gdk_pixbuf_scale_simple (buf, w, h, GDK_INTERP_BILINEAR);
+	}
+	
+	return scaled;
+}
+
