@@ -68,6 +68,10 @@ guint     gnumeric_ascii_strcase_hash  (gconstpointer v);
 gint      gnumeric_ascii_strcase_equal (gconstpointer v, gconstpointer v2);
 gint      gnumeric_utf8_collate_casefold (const char *a, const char *b);
 
+const char *gnm_guess_encoding (const char *raw, size_t len,
+				const char *user_guess,
+				char **utf8_str);
+
 /**
  * System and user paths
  */
@@ -84,8 +88,6 @@ const char *gnm_strunescape (GString *target, const char *string);
 
 char *    gnumeric_utf8_strcapital (const char *p, ssize_t len);
 
-gboolean  gnumeric_valid_filename (const char *filename);
-
 gnm_mem_chunk *gnm_mem_chunk_new (const char *, size_t, size_t);
 void gnm_mem_chunk_destroy (gnm_mem_chunk *, gboolean);
 gpointer gnm_mem_chunk_alloc (gnm_mem_chunk *);
