@@ -2,11 +2,11 @@
 /*
  * MStyle.c: The guts of the style engine.
  *
- * Author:
+ * Authors:
  *   Michael Meeks <mmeeks@gnu.org>
- *
- * Contributors:
  *   Almer S. Tigelaar <almer@gnome.org>
+ *   Jody Goldberg <jody@gnome.org>
+ *   Morten Welinder <terra@gnome.org>
  */
 #include <gnumeric-config.h>
 #include "gnumeric.h"
@@ -20,9 +20,26 @@
 #include "format.h"
 #include "sheet-style.h"
 #include "application.h"
+#include "gutils.h"
 
 #include <stdio.h>
 
+#ifndef USE_MSTYLE_POOL
+#define USE_MSTYLE_POOL 1
+#endif
+
+#if USE_MSTYLE_POOL
+/* Memory pool for mstyles.  */
+static gnm_mem_chunk *mstyle_pool;
+#define CHUNK_ALLOC(T,p) ((T*)gnm_mem_chunk_alloc (p))
+#define CHUNK_ALLOC0(T,p) ((T*)gnm_mem_chunk_alloc0 (p))
+#define CHUNK_FREE(p,v) gnm_mem_chunk_free ((p), (v))
+#else
+#define CHUNK_ALLOC(T,c) g_new (T,1)
+#define CHUNK_ALLOC0(T,c) g_new0 (T,1)
+#define CHUNK_FREE(p,v) g_free ((v))
+#endif
+
 typedef struct {
 	MStyleElementType type;
 	union {
@@ -83,6 +100,10 @@ struct _MStyle {
 	guint32        link_count;
 	Sheet	      *linked_sheet;
 	MStyleElement  elements[MSTYLE_ELEMENT_MAX];
+	PangoAttrList *pango_attrs;
+	double         pango_attrs_zoom;
+	StyleFont     *font;
+	double         font_zoom;
 };
 
 #define MSTYLE_ANY_COLOR             MSTYLE_COLOR_FORE: \
@@ -160,19 +181,20 @@ const char *mstyle_names[MSTYLE_ELEMENT_MAX] = {
 #define d(arg)	do { } while (0)
 #endif
 
-guint
-mstyle_hash (gconstpointer st)
+static guint
+mstyle_hash_internal (gconstpointer st, int i)
 {
 	const MStyle *mstyle = (const MStyle *)st;
-	int     i;
 	guint32 hash = 0;
 
-	for (i = MSTYLE_ELEMENT_CONFLICT + 1; i < MSTYLE_ELEMENT_MAX; i++) {
+	while (i-- > (MSTYLE_ELEMENT_CONFLICT + 1)) {
 		const MStyleElement *e = &mstyle->elements[i];
 		hash = (hash << 7) ^ (hash >> (sizeof (hash) * 8 - 7));
 		switch (i) {
 		case MSTYLE_ANY_COLOR:
-			hash = hash ^ GPOINTER_TO_UINT (e->u.color.any);
+			/* auto colours break things */
+			if (!e->u.color.any->is_auto)
+				hash = hash ^ GPOINTER_TO_UINT (e->u.color.any);
 			break;
 		case MSTYLE_ANY_BORDER:
 			hash = hash ^ GPOINTER_TO_UINT (e->u.border.any);
@@ -218,10 +240,23 @@ mstyle_hash (gconstpointer st)
 	return hash;
 }
 
+guint
+mstyle_hash_XL (gconstpointer st)
+{
+	return mstyle_hash_internal (st, MSTYLE_VALIDATION);
+}
+
+guint
+mstyle_hash (gconstpointer st)
+{
+	return mstyle_hash_internal (st, MSTYLE_ELEMENT_MAX);
+}
+
+
 static char *
 mstyle_element_dump (const MStyleElement *e)
 {
-	GString *ans = g_string_new ("");
+	GString *ans = g_string_new (NULL);
 	char    *txt_ans;
 
 	/* This leaks ans from above.  Let's consider that a feature.  */
@@ -233,15 +268,15 @@ mstyle_element_dump (const MStyleElement *e)
 		break;
 	case MSTYLE_COLOR_BACK:
 		g_string_printf (ans, "\tbackground col %hx:%hx:%hx\n",
-				 e->u.color.any->red,
-				 e->u.color.any->green,
-				 e->u.color.any->blue);
+				 e->u.color.any->color.red,
+				 e->u.color.any->color.green,
+				 e->u.color.any->color.blue);
 		break;
 	case MSTYLE_COLOR_PATTERN:
 		g_string_printf (ans, "\tpattern col %hx:%hx:%hx\n",
-				 e->u.color.any->red,
-				 e->u.color.any->green,
-				 e->u.color.any->blue);
+				 e->u.color.any->color.red,
+				 e->u.color.any->color.green,
+				 e->u.color.any->color.blue);
 		break;
 	case MSTYLE_BORDER_TOP:
 	case MSTYLE_BORDER_BOTTOM:
@@ -261,9 +296,9 @@ mstyle_element_dump (const MStyleElement *e)
 
 	case MSTYLE_COLOR_FORE:
 		g_string_printf (ans, "\tforegnd col %hx:%hx:%hx\n",
-				 e->u.color.any->red,
-				 e->u.color.any->green,
-				 e->u.color.any->blue);
+				 e->u.color.any->color.red,
+				 e->u.color.any->color.green,
+				 e->u.color.any->color.blue);
 		break;
 	case MSTYLE_FONT_NAME:
 		g_string_printf (ans, "\tname '%s'\n", e->u.font.name->str);
@@ -368,7 +403,8 @@ mstyle_element_equal (MStyleElement const *a,
 
 	switch (a->type) {
 	case MSTYLE_ANY_COLOR:
-		return (a->u.color.fore == b->u.color.fore);
+		return (a->u.color.any == b->u.color.any ||
+			(a->u.color.any->is_auto && b->u.color.any->is_auto));
 	case MSTYLE_ANY_BORDER:
 		return (a->u.border.any == b->u.border.any);
 	case MSTYLE_PATTERN:
@@ -455,7 +491,7 @@ mstyle_element_unref (MStyleElement e)
 {
 	switch (e.type) {
 	case MSTYLE_ANY_COLOR:
-		style_color_unref (e.u.color.fore);
+		style_color_unref (e.u.color.any);
 		break;
 	case MSTYLE_ANY_BORDER:
 		style_border_unref (e.u.border.any);
@@ -550,14 +586,36 @@ mstyle_elements_copy (MStyle *new_style, const MStyle *old_style)
 	}
 }
 
+static inline void
+mstyle_pango_clear (MStyle *mstyle)
+{
+	if (mstyle->pango_attrs) {
+		pango_attr_list_unref (mstyle->pango_attrs);
+		mstyle->pango_attrs = NULL;
+	}
+}
+
+
+static inline void
+mstyle_font_clear (MStyle *mstyle)
+{
+	if (mstyle->font) {
+		style_font_unref (mstyle->font);
+		mstyle->font = NULL;
+	}
+}
+
+
 MStyle *
 mstyle_new (void)
 {
-	MStyle *style = g_new0 (MStyle, 1);
+	MStyle *style = CHUNK_ALLOC0 (MStyle, mstyle_pool);
 
 	style->ref_count = 1;
 	style->link_count = 0;
 	style->linked_sheet = NULL;
+	style->pango_attrs = NULL;
+	style->font = NULL;
 	d(("new %p\n", style));
 
 	return style;
@@ -566,13 +624,20 @@ mstyle_new (void)
 MStyle *
 mstyle_copy (const MStyle *style)
 {
-	MStyle *new_style = g_new (MStyle, 1);
+	MStyle *new_style = CHUNK_ALLOC (MStyle, mstyle_pool);
 
 	new_style->ref_count = 1;
 	new_style->link_count = 0;
 	new_style->linked_sheet = NULL;
 	mstyle_elements_copy (new_style, style);
 
+	if ((new_style->pango_attrs = style->pango_attrs))
+		pango_attr_list_ref (new_style->pango_attrs);
+	if ((new_style->font = style->font)) {
+		style_font_ref (new_style->font);
+		new_style->font_zoom = style->font_zoom;
+	}
+
 	d(("copy %p\n", new_style));
 	return new_style;
 }
@@ -581,7 +646,7 @@ MStyle *
 mstyle_copy_merge (const MStyle *orig, const MStyle *overlay)
 {
 	int i;
-	MStyle *res = g_new0 (MStyle, 1);
+	MStyle *res = CHUNK_ALLOC0 (MStyle, mstyle_pool);
 
 	MStyleElement       *res_e;
 	const MStyleElement *orig_e;
@@ -595,8 +660,8 @@ mstyle_copy_merge (const MStyle *orig, const MStyle *overlay)
 	overlay_e = overlay->elements;
 
 	for (i = 0; i < MSTYLE_ELEMENT_MAX; i++)
-		res_e [i] = mstyle_element_ref (
-			(overlay_e [i].type ? overlay_e : orig_e) + i);
+		res_e[i] = mstyle_element_ref (
+			(overlay_e[i].type ? overlay_e : orig_e) + i);
 
 	d(("copy merge %p\n", res));
 	return res;
@@ -703,8 +768,10 @@ mstyle_unref (MStyle *style)
 
 		if (style->elements)
 			mstyle_elements_unref (style->elements);
+		mstyle_pango_clear (style);
+		mstyle_font_clear (style);
 
-		g_free (style);
+		CHUNK_FREE (mstyle_pool, style);
 	}
 }
 
@@ -806,11 +873,13 @@ MStyle *
 mstyle_link_sheet (MStyle *style, Sheet *sheet)
 {
 	StyleColor *auto_color;
-	MStyle *orig = style;
+	gboolean style_is_orig = TRUE;
 
 	if (style->linked_sheet != NULL) {
+		MStyle *orig = style;
 		style = mstyle_copy (style);
 		mstyle_unref (orig);
+		style_is_orig = FALSE;
 
 		/* safety test */
 		g_return_val_if_fail (style->linked_sheet != sheet, style);
@@ -821,8 +890,8 @@ mstyle_link_sheet (MStyle *style, Sheet *sheet)
 
 	auto_color = sheet_style_get_auto_pattern_color (sheet);
 	if (mstyle_is_element_set (style, MSTYLE_COLOR_PATTERN))
-		style = link_pattern_color (style, auto_color, style == orig);
-	style = link_border_colors (style, auto_color, style == orig);
+		style = link_pattern_color (style, auto_color, style_is_orig);
+	style = link_border_colors (style, auto_color, style_is_orig);
 	style_color_unref (auto_color);
 
 	style->linked_sheet = sheet;
@@ -972,7 +1041,6 @@ mstyle_equal_XL (const MStyle *a, const MStyle *b)
 	return TRUE;
 }
 
-
 gboolean
 mstyle_empty (const MStyle *style)
 {
@@ -1065,6 +1133,7 @@ mstyle_set_color (MStyle *st, MStyleElementType t,
 		mstyle_element_unref (st->elements[t]);
 		st->elements[t].type = t;
 		st->elements[t].u.color.any = col;
+		mstyle_pango_clear (st);
 		break;
 	default:
 		g_warning ("Not a color element");
@@ -1140,39 +1209,45 @@ mstyle_get_pattern (const MStyle *style)
 }
 
 StyleFont *
-mstyle_get_font (const MStyle *style, double zoom)
+mstyle_get_font (const MStyle *style, PangoContext *context, double zoom)
 {
-	StyleFont *font;
-	const gchar *name;
-	gboolean bold, italic;
-	double size;
-
 	g_return_val_if_fail (style != NULL, NULL);
 
-	if (mstyle_is_element_set (style, MSTYLE_FONT_NAME))
-		name = mstyle_get_font_name (style);
-	else
-		name = DEFAULT_FONT;
+	if (!style->font || style->font_zoom != zoom) {
+		const gchar *name;
+		gboolean bold, italic;
+		double size;
 
-	if (mstyle_is_element_set (style, MSTYLE_FONT_BOLD))
-		bold = mstyle_get_font_bold (style);
-	else
-		bold = FALSE;
+		mstyle_font_clear ((MStyle *)style);
 
-	if (mstyle_is_element_set (style, MSTYLE_FONT_ITALIC))
-		italic = mstyle_get_font_italic (style);
-	else
-		italic = FALSE;
+		if (mstyle_is_element_set (style, MSTYLE_FONT_NAME))
+			name = mstyle_get_font_name (style);
+		else
+			name = DEFAULT_FONT;
 
-	if (mstyle_is_element_set (style, MSTYLE_FONT_SIZE))
-		size = mstyle_get_font_size (style);
-	else
-		size = DEFAULT_SIZE;
+		if (mstyle_is_element_set (style, MSTYLE_FONT_BOLD))
+			bold = mstyle_get_font_bold (style);
+		else
+			bold = FALSE;
 
-	font = style_font_new (
-		name, size, zoom, bold, italic);
+		if (mstyle_is_element_set (style, MSTYLE_FONT_ITALIC))
+			italic = mstyle_get_font_italic (style);
+		else
+			italic = FALSE;
 
-	return font;
+		if (mstyle_is_element_set (style, MSTYLE_FONT_SIZE))
+			size = mstyle_get_font_size (style);
+		else
+			size = DEFAULT_SIZE;
+
+		((MStyle *)style)->font =
+			style_font_new (context, name, size,
+					zoom, bold, italic);
+		((MStyle *)style)->font_zoom = zoom;
+	}
+
+	style_font_ref (style->font);
+	return style->font;
 }
 
 void
@@ -1184,6 +1259,8 @@ mstyle_set_font_name (MStyle *style, const char *name)
 	mstyle_element_unref (style->elements[MSTYLE_FONT_NAME]);
 	style->elements[MSTYLE_FONT_NAME].type = MSTYLE_FONT_NAME;
 	style->elements[MSTYLE_FONT_NAME].u.font.name = string_get (name);
+	mstyle_font_clear (style);
+	mstyle_pango_clear (style);
 }
 
 const char *
@@ -1201,6 +1278,8 @@ mstyle_set_font_bold (MStyle *style, gboolean bold)
 
 	style->elements[MSTYLE_FONT_BOLD].type = MSTYLE_FONT_BOLD;
 	style->elements[MSTYLE_FONT_BOLD].u.font.bold = bold;
+	mstyle_font_clear (style);
+	mstyle_pango_clear (style);
 }
 
 gboolean
@@ -1218,6 +1297,8 @@ mstyle_set_font_italic (MStyle *style, gboolean italic)
 
 	style->elements[MSTYLE_FONT_ITALIC].type = MSTYLE_FONT_ITALIC;
 	style->elements[MSTYLE_FONT_ITALIC].u.font.italic = italic;
+	mstyle_font_clear (style);
+	mstyle_pango_clear (style);
 }
 
 gboolean
@@ -1235,6 +1316,7 @@ mstyle_set_font_uline (MStyle *style, StyleUnderlineType const underline)
 
 	style->elements[MSTYLE_FONT_UNDERLINE].type = MSTYLE_FONT_UNDERLINE;
 	style->elements[MSTYLE_FONT_UNDERLINE].u.font.underline = underline;
+	mstyle_pango_clear (style);
 }
 
 StyleUnderlineType
@@ -1252,6 +1334,7 @@ mstyle_set_font_strike (MStyle *style, gboolean const strikethrough)
 
 	style->elements[MSTYLE_FONT_STRIKETHROUGH].type = MSTYLE_FONT_STRIKETHROUGH;
 	style->elements[MSTYLE_FONT_STRIKETHROUGH].u.font.strikethrough = strikethrough;
+	mstyle_pango_clear (style);
 }
 
 gboolean
@@ -1269,6 +1352,8 @@ mstyle_set_font_size (MStyle *style, double size)
 
 	style->elements[MSTYLE_FONT_SIZE].type = MSTYLE_FONT_SIZE;
 	style->elements[MSTYLE_FONT_SIZE].u.font.size = size;
+	mstyle_font_clear (style);
+	mstyle_pango_clear (style);
 }
 
 double
@@ -1398,7 +1483,24 @@ mstyle_get_wrap_text (const MStyle *style)
 {
 	g_return_val_if_fail (mstyle_is_element_set (style, MSTYLE_WRAP_TEXT), FALSE);
 
-	return style->elements [MSTYLE_WRAP_TEXT].u.wrap_text;
+	return style->elements[MSTYLE_WRAP_TEXT].u.wrap_text;
+}
+
+/*
+ * Same as mstyle_get_wrap_text except that if either halign or valign
+ * is _JUSTIFY, the result will be TRUE.
+ */
+gboolean
+mstyle_get_effective_wrap_text (const MStyle *style)
+{
+	g_return_val_if_fail (mstyle_is_element_set (style, MSTYLE_WRAP_TEXT), FALSE);
+	g_return_val_if_fail (mstyle_is_element_set (style, MSTYLE_ALIGN_V), FALSE);
+	g_return_val_if_fail (mstyle_is_element_set (style, MSTYLE_ALIGN_H), FALSE);
+
+	/* Note: HALIGN_GENERAL never expands to HALIGN_JUSTIFY.  */
+	return (style->elements[MSTYLE_WRAP_TEXT].u.wrap_text ||
+		style->elements[MSTYLE_ALIGN_V].u.align.v == VALIGN_JUSTIFY ||
+		style->elements[MSTYLE_ALIGN_H].u.align.h == HALIGN_JUSTIFY);
 }
 
 void
@@ -1415,7 +1517,7 @@ mstyle_get_shrink_to_fit (const MStyle *style)
 {
 	g_return_val_if_fail (mstyle_is_element_set (style, MSTYLE_SHRINK_TO_FIT), FALSE);
 
-	return style->elements [MSTYLE_SHRINK_TO_FIT].u.wrap_text;
+	return style->elements[MSTYLE_SHRINK_TO_FIT].u.wrap_text;
 }
 void
 mstyle_set_content_locked (MStyle *style, gboolean f)
@@ -1431,7 +1533,7 @@ mstyle_get_content_locked (const MStyle *style)
 {
 	g_return_val_if_fail (mstyle_is_element_set (style, MSTYLE_CONTENT_LOCKED), FALSE);
 
-	return style->elements [MSTYLE_CONTENT_LOCKED].u.content_locked;
+	return style->elements[MSTYLE_CONTENT_LOCKED].u.content_locked;
 }
 void
 mstyle_set_content_hidden (MStyle *style, gboolean f)
@@ -1447,7 +1549,7 @@ mstyle_get_content_hidden (const MStyle *style)
 {
 	g_return_val_if_fail (mstyle_is_element_set (style, MSTYLE_CONTENT_HIDDEN), FALSE);
 
-	return style->elements [MSTYLE_CONTENT_HIDDEN].u.content_hidden;
+	return style->elements[MSTYLE_CONTENT_HIDDEN].u.content_hidden;
 }
 
 void
@@ -1522,20 +1624,35 @@ mstyle_visible_in_blank (const MStyle *st)
 }
 
 PangoAttrList *
-mstyle_get_pango_attrs (const MStyle *mstyle, const StyleColor *fore)
+mstyle_get_pango_attrs (const MStyle *mstyle,
+			PangoContext *context,
+			double zoom)
 {
 	PangoAttribute *attr;
-	PangoAttrList *res = pango_attr_list_new ();
+	PangoAttrList *res;
 
-	/* Set the font colour */
-	if (fore == NULL)
-		fore = mstyle_get_color (mstyle, MSTYLE_COLOR_FORE);
-	attr = pango_attr_foreground_new (fore->red, fore->green, fore->blue);
-	attr->start_index = 0;
-	attr->end_index = -1;
-	pango_attr_list_insert (res, attr);
+	if (mstyle->pango_attrs) {
+		if (zoom == mstyle->pango_attrs_zoom) {
+			pango_attr_list_ref (mstyle->pango_attrs);
+			return mstyle->pango_attrs;
+		}
+		pango_attr_list_unref (((MStyle *)mstyle)->pango_attrs);
+	}
 
-	/* Handle underlining */
+	((MStyle *)mstyle)->pango_attrs = res = pango_attr_list_new ();
+	((MStyle *)mstyle)->pango_attrs_zoom = zoom;
+
+	/* Foreground colour.  */
+	/* See http://bugzilla.gnome.org/show_bug.cgi?id=105322 */
+	if (0) {
+		const StyleColor *fore = mstyle_get_color (mstyle, MSTYLE_COLOR_FORE);
+		attr = pango_attr_foreground_new (fore->color.red, fore->color.green, fore->color.blue);
+		attr->start_index = 0;
+		attr->end_index = -1;
+		pango_attr_list_insert (res, attr);
+	}
+
+	/* Handle underlining.  */
 	switch (mstyle_get_font_uline (mstyle)) {
 	case UNDERLINE_SINGLE :
 		attr = pango_attr_underline_new (PANGO_UNDERLINE_SINGLE);
@@ -1555,13 +1672,57 @@ mstyle_get_pango_attrs (const MStyle *mstyle, const StyleColor *fore)
 		break;
 	};
 
-	/* Handle strikethrough */
-	if (mstyle_get_font_strike (mstyle)){
+	/* Handle strike-through.  */
+	if (mstyle_get_font_strike (mstyle)) {
 		attr = pango_attr_strikethrough_new (TRUE);
 		attr->start_index = 0;
 		attr->end_index = -1;
 		pango_attr_list_insert (res, attr);
 	}
 
+	/* Handle font.  */
+	{
+		StyleFont *font = mstyle_get_font (mstyle, context, zoom);
+		attr = pango_attr_font_desc_new (font->pango.font_descr);
+		attr->start_index = 0;
+		attr->end_index = -1;
+		pango_attr_list_insert (res, attr);
+		style_font_unref (font);
+	}
+
+	pango_attr_list_ref (res);
 	return res;
 }
+
+/* ------------------------------------------------------------------------- */
+
+void
+mstyle_init (void)
+{
+#if USE_MSTYLE_POOL
+	mstyle_pool =
+		gnm_mem_chunk_new ("mstyle pool",
+				   sizeof (MStyle),
+				   16 * 1024 - 128);
+#endif
+}
+
+#if USE_MSTYLE_POOL
+static void
+cb_mstyle_pool_leak (gpointer data, gpointer user)
+{
+	MStyle *mstyle = data;
+	fprintf (stderr, "Leaking mstyle at %p.\n", mstyle);
+	mstyle_dump (mstyle);
+}
+#endif
+
+void
+mstyle_shutdown (void)
+{
+#if USE_MSTYLE_POOL
+	gnm_mem_chunk_foreach_leak (mstyle_pool, cb_mstyle_pool_leak, NULL);
+	gnm_mem_chunk_destroy (mstyle_pool, FALSE);
+	mstyle_pool = NULL;
+#endif
+}
