@@ -24,6 +24,7 @@
 
 #include "workbook.h"
 #include "sheet.h"
+#include "sheet-private.h"
 #include "cell.h"
 #include "expr.h"
 #include "value.h"
@@ -315,8 +316,7 @@ static Value *
 cb_collect_unique (Sheet *sheet, int col, int row, Cell *cell,
 		   UniqueCollection *uc)
 {
-	if (cell_is_blank (cell) ||
-	    (cell->value->type == VALUE_STRING && *(cell->value->v_str.val->str) == '\0'))
+	if (cell_is_blank (cell))
 		uc->has_blank = TRUE;
 	else
 		g_hash_table_replace (uc->hash, cell->value, cell->value);
@@ -376,6 +376,7 @@ collect_unique_elements (GnmFilterField *field,
 	/* r.end.row =  XL actually extend to the first non-empty element in the list */
 	r.end.col = r.start.col += filter_field_index (field);
 	uc.has_blank = FALSE;
+#warning "FIXME: is this really the type of equality (and hash) we want here?"
 	uc.hash = g_hash_table_new (
 			(GHashFunc) value_hash, (GEqualFunc) value_equal);
 	sheet_foreach_cell_in_range (field->filter->sheet,
@@ -618,6 +619,8 @@ filter_expr_init (FilterExpr *fexpr, unsigned i,
 		  GnmFilter const *filter)
 {
 	Value *tmp = cond->value[i];
+	fexpr->date_conv = workbook_date_conv (filter->sheet->workbook);
+
 	if (VALUE_IS_STRING (tmp)) {
 		GnmFilterOp op = cond->op[i];
 		char const *str = value_peek_string (tmp);
@@ -629,7 +632,6 @@ filter_expr_init (FilterExpr *fexpr, unsigned i,
 			return;
 	}
 	fexpr->val[i] = value_duplicate (tmp);
-	fexpr->date_conv = workbook_date_conv (filter->sheet->workbook);
 }
 
 static void
@@ -702,19 +704,17 @@ cb_filter_expr (Sheet *sheet, int col, int row, Cell *cell,
 /*****************************************************************************/
 
 static Value *
-cb_filter_blanks (Sheet *sheet, int col, int row, Cell *cell, gpointer data)
+cb_filter_non_blanks (Sheet *sheet, int col, int row, Cell *cell, gpointer data)
 {
-	if (cell_is_blank (cell) ||
-	    (cell->value->type == VALUE_STRING && *(cell->value->v_str.val->str) == '\0'))
+	if (cell_is_blank (cell))
 		colrow_set_visibility (sheet, FALSE, FALSE, row, row);
 	return NULL;
 }
 
 static Value *
-cb_filter_non_blanks (Sheet *sheet, int col, int row, Cell *cell, gpointer data)
+cb_filter_blanks (Sheet *sheet, int col, int row, Cell *cell, gpointer data)
 {
-	if (!cell_is_blank (cell) &&
-	    !(cell->value->type == VALUE_STRING && *(cell->value->v_str.val->str) == '\0'))
+	if (!cell_is_blank (cell))
 		colrow_set_visibility (sheet, FALSE, FALSE, row, row);
 	return NULL;
 }
@@ -916,6 +916,7 @@ gnm_filter_add_field (GnmFilter *filter, int i)
 		SO_ANCHOR_PERCENTAGE_FROM_COLROW_END
 	};
 	static float const offsets [4] = { .0, .0, 0., 0. };
+	int n;
 	Range tmp;
 	SheetObjectAnchor anchor;
 	GnmFilterField *field = g_object_new (filter_field_get_type (), NULL);
@@ -927,7 +928,12 @@ gnm_filter_add_field (GnmFilter *filter, int i)
 				  SO_DIR_DOWN_RIGHT);
 	sheet_object_anchor_set (&field->parent, &anchor);
 	sheet_object_set_sheet (&field->parent, filter->sheet);
-	g_ptr_array_add (filter->fields, field);
+
+	g_ptr_array_add (filter->fields, NULL);
+	for (n = filter->fields->len; --n > i ; )
+		g_ptr_array_index (filter->fields, n) =
+			g_ptr_array_index (filter->fields, n-1);
+	g_ptr_array_index (filter->fields, n) = field;
 	g_object_unref (G_OBJECT (field));
 }
 
@@ -958,6 +964,7 @@ gnm_filter_new (Sheet *sheet, Range const *r)
 		gnm_filter_add_field (filter, i);
 
 	sheet->filters = g_slist_prepend (sheet->filters, filter);
+	sheet->priv->filters_changed = TRUE;
 
 	return filter;
 }
@@ -984,8 +991,10 @@ gnm_filter_remove (GnmFilter *filter)
 	int i;
 
 	g_return_if_fail (filter != NULL);
+	g_return_if_fail (filter->sheet != NULL);
 
 	sheet = filter->sheet;
+	sheet->priv->filters_changed = TRUE;
 	sheet->filters = g_slist_remove (sheet->filters, filter);
 	for (i = filter->r.start.row; ++i <= filter->r.end.row ; ) {
 		ColRowInfo *ri = sheet_row_get (sheet, i);
@@ -1125,33 +1134,42 @@ sheet_filter_insdel_colrow (Sheet *sheet, gboolean is_cols, gboolean is_insert,
 		filter = ptr->data;
 
 		if (is_cols) {
-			if (start > filter->r.end.col)
+			if (start > filter->r.end.col)	/* a */
 				continue;
 			if (is_insert) {
 				filter->r.end.col += count;
-				if (start >= filter->r.start.col) {
+				/* inserting in the middle of a filter adds
+				 * fields.  Everything else just moves it */
+				if (start > filter->r.start.col &&
+				    start <= filter->r.end.col) {
 					while (count--)
 						gnm_filter_add_field (filter,
 							start - filter->r.start.col + count);
 				} else
 					filter->r.start.col += count;
 			} else {
-				if (start <= filter->r.start.col) {
-					filter->r.end.col -= count;
-					if ((start+count) <= filter->r.start.col)
-						filter->r.start.col -= count;
+				int start_del = start - filter->r.start.col;
+				int end_del   = start_del + count;
+				if (start_del <= 0) {
+					start_del = 0;
+					if (end_del > 0)
+						filter->r.start.col = start;	/* c */
 					else
-						filter->r.start.col = start;
-				} else if ((start+count) > filter->r.end.col)
-					filter->r.end.col = start -1;
-				else
+						filter->r.start.col -= count;	/* b */
 					filter->r.end.col -= count;
+				} else {
+					if ((unsigned)end_del > filter->fields->len) {
+						end_del = filter->fields->len;
+						filter->r.end.col = start - 1;	/* d */
+					} else
+						filter->r.end.col -= count;
+				}
 
-				if (filter->r.end.col < filter->r.end.col)
+				if (filter->r.end.col < filter->r.start.col)
 					filter = NULL;
 				else
-					g_ptr_array_set_size (filter->fields,
-						range_width (&filter->r));
+					while (end_del-- > start_del)
+						g_ptr_array_remove_index (filter->fields, end_del);
 			}
 		} else {
 			if (start > filter->r.end.row)
@@ -1186,5 +1204,7 @@ sheet_filter_insdel_colrow (Sheet *sheet, gboolean is_cols, gboolean is_insert,
 			gnm_filter_free (filter);
 		}
 	}
+	if (filters != NULL)
+		sheet->priv->filters_changed = TRUE;
 	g_slist_free (filters);
 }
