@@ -19,14 +19,17 @@
 #include "style-color.h"
 #include "sheet-object-impl.h"
 #include "workbook-edit.h"
+#include "dialogs/help.h"
 
 #include <gdk/gdkkeysyms.h>
 #include <gsf/gsf-impl-utils.h>
-#include <gal/widgets/widget-color-combo.h>
+#include <widgets/widget-color-combo.h>
 #include <libfoocanvas/foo-canvas-line.h>
 #include <libfoocanvas/foo-canvas-rect-ellipse.h>
 #include <libfoocanvas/foo-canvas-polygon.h>
 #include <libfoocanvas/foo-canvas-text.h>
+
+#include <libart_lgpl/art_affine.h>
 #include <math.h>
 
 #define SHEET_OBJECT_CONFIG_KEY "sheet-object-arrow-key"
@@ -281,142 +284,116 @@ sheet_object_graphic_clone (SheetObject const *so, Sheet *sheet)
 
 static void
 sheet_object_graphic_print (SheetObject const *so, GnomePrintContext *ctx,
-			    double base_x, double base_y)
+			    double width, double height)
 {
-	SheetObjectGraphic *sog;
-	double coords [4];
-	double x1 = 0.0, y1 = 0.0, x2 = 0.0, y2 = 0.0;
+	SheetObjectGraphic *sog = SHEET_OBJECT_GRAPHIC (so);
+	double x1, y1, x2, y2;
 
-	g_return_if_fail (IS_SHEET_OBJECT_GRAPHIC (so));
-	g_return_if_fail (GNOME_IS_PRINT_CONTEXT (ctx));
-	sog = SHEET_OBJECT_GRAPHIC (so);
+	if (sog->fill_color == NULL)
+		return;
 
-	sheet_object_position_pts_get (so, coords);
-
-	gnome_print_gsave (ctx);
-
-	if (sog->fill_color) {
-		switch (so->anchor.direction) {
-		case SO_DIR_UP_RIGHT:
-		case SO_DIR_DOWN_RIGHT:
-			x1 = base_x;
-			x2 = base_x + (coords [2] - coords [0]);
-			break;
-		case SO_DIR_UP_LEFT:
-		case SO_DIR_DOWN_LEFT:
-			x1 = base_x + (coords [2] - coords [0]);
-			x2 = base_x;
-			break;
-		default:
-			g_warning ("Cannot guess direction!");
-			gnome_print_grestore (ctx);
-			return;
-		}
-
-		switch (so->anchor.direction) {
-		case SO_DIR_UP_LEFT:
-		case SO_DIR_UP_RIGHT:
-			y1 = base_y;
-			y2 = base_y + (coords [3] - coords [1]);
-			break;
-		case SO_DIR_DOWN_LEFT:
-		case SO_DIR_DOWN_RIGHT:
-			y1 = base_y + (coords [3] - coords [1]);
-			y2 = base_y;
-			break;
-		default:
-			g_warning ("Cannot guess direction!");
-			gnome_print_grestore (ctx);
-			return;
-		}
-
-		gnome_print_setrgbcolor (ctx,
-			sog->fill_color->red   / (double) 0xffff,
-			sog->fill_color->green / (double) 0xffff,
-			sog->fill_color->blue  / (double) 0xffff);
-
-		if (sog->type == SHEET_OBJECT_ARROW) {
-			double phi;
-
-			phi = atan2 (y2 - y1, x2 - x1) - M_PI_2;
-
-			gnome_print_gsave (ctx);
-			gnome_print_translate (ctx, x2, y2);
-			gnome_print_rotate (ctx, phi / (2 * M_PI) * 360);
-			gnome_print_setlinewidth (ctx, 1.0);
-			gnome_print_newpath (ctx);
-			gnome_print_moveto (ctx, 0.0, 0.0);
-			gnome_print_lineto (ctx, -sog->c, -sog->b);
-			gnome_print_lineto (ctx, 0.0, -sog->a);
-			gnome_print_lineto (ctx, sog->c, -sog->b);
-			gnome_print_closepath (ctx);
-			gnome_print_fill (ctx);
-			gnome_print_grestore (ctx);
-
-			/*
-			 * Make the line shorter so that the arrow won't be
-			 * on top of a (perhaps quite fat) line.
-			 */
-			x2 += sog->a * sin (phi);
-			y2 -= sog->a * cos (phi);
-		}
-
-		gnome_print_setlinewidth (ctx, sog->width);
-		gnome_print_newpath (ctx);
-		gnome_print_moveto (ctx, x1, y1);
-		gnome_print_lineto (ctx, x2, y2);
-		gnome_print_stroke (ctx);
+	switch (so->anchor.direction) {
+	case SO_DIR_UP_RIGHT:
+	case SO_DIR_DOWN_RIGHT:
+		x1 = 0.;
+		x2 = width;
+		break;
+	case SO_DIR_UP_LEFT:
+	case SO_DIR_DOWN_LEFT:
+		x1 = width;
+		x2 = 0.;
+		break;
+	default:
+		g_warning ("Cannot guess direction!");
+		return;
 	}
 
-	gnome_print_grestore (ctx);
+	switch (so->anchor.direction) {
+	case SO_DIR_UP_LEFT:
+	case SO_DIR_UP_RIGHT:
+		y1 = -height;
+		y2 = 0.;
+		break;
+	case SO_DIR_DOWN_LEFT:
+	case SO_DIR_DOWN_RIGHT:
+		y1 = 0.;
+		y2 = -height;
+		break;
+	default:
+		g_warning ("Cannot guess direction!");
+		return;
+	}
+
+	gnome_print_setrgbcolor (ctx,
+		sog->fill_color->color.red   / (double) 0xffff,
+		sog->fill_color->color.green / (double) 0xffff,
+		sog->fill_color->color.blue  / (double) 0xffff);
+
+	if (sog->type == SHEET_OBJECT_ARROW) {
+		double phi;
+
+		phi = atan2 (y2 - y1, x2 - x1) - M_PI_2;
+
+		gnome_print_gsave (ctx);
+		gnome_print_translate (ctx, x2, y2);
+		gnome_print_rotate (ctx, phi / (2 * M_PI) * 360);
+		gnome_print_setlinewidth (ctx, 1.0);
+		gnome_print_newpath (ctx);
+		gnome_print_moveto (ctx, 0.0, 0.0);
+		gnome_print_lineto (ctx, -sog->c, -sog->b);
+		gnome_print_lineto (ctx, 0.0, -sog->a);
+		gnome_print_lineto (ctx, sog->c, -sog->b);
+		gnome_print_closepath (ctx);
+		gnome_print_fill (ctx);
+		gnome_print_grestore (ctx);
+
+		/*
+		 * Make the line shorter so that the arrow won't be
+		 * on top of a (perhaps quite fat) line.
+		 */
+		x2 += sog->a * sin (phi);
+		y2 -= sog->a * cos (phi);
+	}
+
+	gnome_print_setlinewidth (ctx, sog->width);
+	gnome_print_newpath (ctx);
+	gnome_print_moveto (ctx, x1, y1);
+	gnome_print_lineto (ctx, x2, y2);
+	gnome_print_stroke (ctx);
 }
 
-typedef struct
-{
+typedef struct {
 	GladeXML           *gui;
 	GtkWidget          *dialog;
-	GtkWidget *canvas;
-	FooCanvasItem *arrow;
-	GtkWidget *fill_color_combo;
-	GtkSpinButton *spin_arrow_tip;
-	GtkSpinButton *spin_arrow_length;
-	GtkSpinButton *spin_arrow_width;
-	GtkSpinButton *spin_line_width;
+	GtkWidget	*canvas;
+	FooCanvasItem	*arrow;
+	GtkSpinButton	*spin_arrow_tip;
+	GtkSpinButton	*spin_arrow_length;
+	GtkSpinButton	*spin_arrow_width;
+	GtkSpinButton	*spin_line_width;
 
-	StyleColor *fill_color;
-	double width;
-	double a, b, c;                   /* Only for arrows */
+	/* Store the initial values */
+	StyleColor	*fill_color;
+	double		 width, a, b, c;
 
 	WorkbookControlGUI *wbcg;
 	SheetObjectGraphic *sog;
-	Sheet		   *sheet;
 } DialogGraphicData;
 
-
-
-static gboolean
-cb_dialog_graphic_config_destroy (GtkObject *w, DialogGraphicData *state)
+static void
+cb_dialog_graphic_config_destroy (DialogGraphicData *state)
 {
-	g_return_val_if_fail (w != NULL, FALSE);
-	g_return_val_if_fail (state != NULL, FALSE);
-
 	wbcg_edit_detach_guru (state->wbcg);
-
+	if (state->fill_color != NULL) {
+		style_color_unref (state->fill_color);
+		state->fill_color = NULL;
+	}
 	if (state->gui != NULL) {
 		g_object_unref (G_OBJECT (state->gui));
 		state->gui = NULL;
 	}
-
 	state->dialog = NULL;
 	g_free (state);
-
-	return FALSE;
-}
-
-static void
-cb_dialog_graphic_config_ok_clicked (GtkWidget *button, DialogGraphicData *state)
-{
-	gtk_widget_destroy (state->dialog);
 }
 
 static void
@@ -426,7 +403,8 @@ cb_dialog_graphic_config_cancel_clicked (GtkWidget *button, DialogGraphicData *s
 
 	sheet_object_graphic_width_set (state->sog, state->width);
 	sheet_object_graphic_fill_color_set (so,
-					     state->fill_color);
+		state->fill_color);
+	state->fill_color = NULL;
 
 	if (state->sog->type == SHEET_OBJECT_ARROW)
 		sheet_object_graphic_abc_set (state->sog,
@@ -466,14 +444,12 @@ cb_adjustment_value_changed (GtkAdjustment *adj, DialogGraphicData *state)
 }
 
 static void
-cb_fill_color_changed (ColorCombo *color_combo, GdkColor *color,
+cb_fill_color_changed (GtkWidget *cc, GdkColor *color,
 		       gboolean is_custom, gboolean by_user, gboolean is_default,
 		       DialogGraphicData *state)
 {
-	SheetObject *so = SHEET_OBJECT (state->sog);
-	sheet_object_graphic_fill_color_set (so, color_combo_get_style_color (
-						     state->fill_color_combo));
-
+	sheet_object_graphic_fill_color_set (SHEET_OBJECT (state->sog),
+		color_combo_get_style_color (cc));
 	foo_canvas_item_set (state->arrow, "fill_color_gdk", color, NULL);
 }
 
@@ -484,7 +460,7 @@ sheet_object_graphic_user_config (SheetObject *so, SheetControl *sc)
 	WorkbookControlGUI *wbcg = scg_get_wbcg (SHEET_CONTROL_GUI (sc));
 	DialogGraphicData *state;
 	FooCanvasPoints *points;
-	GtkWidget *table;
+	GtkWidget *table, *w;
 
 	g_return_if_fail (sog != NULL);
 
@@ -495,30 +471,34 @@ sheet_object_graphic_user_config (SheetObject *so, SheetControl *sc)
 	state = g_new0 (DialogGraphicData, 1);
 	state->sog = sog;
 	state->wbcg = wbcg;
-	state->sheet = sc_sheet	(sc);
 
 	sog = SHEET_OBJECT_GRAPHIC (so);
-	state->gui = gnumeric_glade_xml_new (wbcg, "so-arrow.glade");
+	state->gui = gnm_glade_xml_new (COMMAND_CONTEXT (wbcg),
+		"so-arrow.glade", NULL, NULL);
 	state->dialog = glade_xml_get_widget (state->gui, "SO-Arrow");
 
  	table = glade_xml_get_widget (state->gui, "table");
 	state->canvas = foo_canvas_new ();
+	g_object_set (G_OBJECT (state->canvas), "can_focus", FALSE, NULL);
 	gtk_table_attach_defaults (GTK_TABLE (table), state->canvas,
 				   2, 3, 0, (sog->type != SHEET_OBJECT_ARROW) ? 2 : 5);
 	gtk_widget_show (GTK_WIDGET (state->canvas));
 
-	state->fill_color_combo = color_combo_new (NULL, NULL, NULL,
-					color_group_fetch ("color", so));
-	color_combo_set_color (COLOR_COMBO (state->fill_color_combo),
-			       sog->fill_color ? &sog->fill_color->color : NULL);
+	w = color_combo_new (NULL, NULL, NULL, color_group_fetch ("color", so));
+	gtk_label_set_mnemonic_widget (
+		GTK_LABEL (glade_xml_get_widget (state->gui, "label_color")), w);
+	color_combo_set_color (COLOR_COMBO (w),
+		sog->fill_color ? &sog->fill_color->color : NULL);
+	color_combo_set_instant_apply (COLOR_COMBO (w), FALSE);
+	gtk_combo_box_set_tearable (GTK_COMBO_BOX (w), FALSE);
 	state->fill_color = style_color_ref (sog->fill_color);
-	gtk_table_attach_defaults (GTK_TABLE (table),
-				   state->fill_color_combo, 1, 2, 0, 1);
-	gtk_combo_box_set_arrow_relief (GTK_COMBO_BOX (state->fill_color_combo), 
-					GTK_RELIEF_NORMAL);
-	color_combo_box_set_preview_relief (COLOR_COMBO (state->fill_color_combo), 
-					    GTK_RELIEF_NORMAL);
-	gtk_widget_show (GTK_WIDGET (state->fill_color_combo));
+	gtk_table_attach_defaults (GTK_TABLE (table), w, 1, 2, 0, 1);
+	gtk_combo_box_set_arrow_relief (GTK_COMBO_BOX (w), GTK_RELIEF_NORMAL);
+	color_combo_box_set_preview_relief (COLOR_COMBO (w), GTK_RELIEF_NORMAL);
+	gtk_widget_show (GTK_WIDGET (w));
+	g_signal_connect (G_OBJECT (w),
+		"color_changed",
+		G_CALLBACK (cb_fill_color_changed), state);
 
 	state->spin_arrow_tip = GTK_SPIN_BUTTON (glade_xml_get_widget (
 						    state->gui, "spin_arrow_tip"));
@@ -537,6 +517,14 @@ sheet_object_graphic_user_config (SheetObject *so, SheetControl *sc)
 	gtk_spin_button_set_value (state->spin_arrow_tip, state->a);
 	gtk_spin_button_set_value (state->spin_arrow_length, state->b);
 	gtk_spin_button_set_value (state->spin_arrow_width, state->c);
+	gnumeric_editable_enters (GTK_WINDOW (state->dialog),
+				  GTK_WIDGET (state->spin_line_width));
+	gnumeric_editable_enters (GTK_WINDOW (state->dialog),
+				  GTK_WIDGET (state->spin_arrow_tip));
+	gnumeric_editable_enters (GTK_WINDOW (state->dialog),
+				  GTK_WIDGET (state->spin_arrow_length));
+	gnumeric_editable_enters (GTK_WINDOW (state->dialog),
+				  GTK_WIDGET (state->spin_arrow_width));
 
 	if (sog->type != SHEET_OBJECT_ARROW) {
 		gtk_widget_hide (glade_xml_get_widget (state->gui, "label_arrow_tip"));
@@ -545,7 +533,7 @@ sheet_object_graphic_user_config (SheetObject *so, SheetControl *sc)
 		gtk_widget_hide (GTK_WIDGET (state->spin_arrow_tip));
 		gtk_widget_hide (GTK_WIDGET (state->spin_arrow_length));
 		gtk_widget_hide (GTK_WIDGET (state->spin_arrow_width));
-	} 
+	}
 	gtk_widget_show (state->dialog);
 
 	points = foo_canvas_points_new (2);
@@ -571,11 +559,11 @@ sheet_object_graphic_user_config (SheetObject *so, SheetControl *sc)
 					0., 0., state->canvas->allocation.width,
 					state->canvas->allocation.height);
 	cb_adjustment_value_changed (NULL, state);
-	g_signal_connect (G_OBJECT 
+	g_signal_connect (G_OBJECT
 			  (gtk_spin_button_get_adjustment (state->spin_arrow_tip)),
 			  "value_changed",
 			  G_CALLBACK (cb_adjustment_value_changed), state);
-	g_signal_connect (G_OBJECT 
+	g_signal_connect (G_OBJECT
 			  (gtk_spin_button_get_adjustment (state->spin_arrow_length)),
 			  "value_changed",
 			  G_CALLBACK (cb_adjustment_value_changed), state);
@@ -583,31 +571,30 @@ sheet_object_graphic_user_config (SheetObject *so, SheetControl *sc)
 			  (gtk_spin_button_get_adjustment (state->spin_arrow_width)),
 			  "value_changed",
 			  G_CALLBACK (cb_adjustment_value_changed), state);
-	g_signal_connect (G_OBJECT (state->fill_color_combo),
-			  "color_changed",
-			  G_CALLBACK (cb_fill_color_changed), state);
 	g_signal_connect (G_OBJECT
 			  (gtk_spin_button_get_adjustment (state->spin_line_width)),
 			  "value_changed",
 			  G_CALLBACK (cb_adjustment_value_changed), state);
-	g_signal_connect (G_OBJECT (state->dialog),
-			  "destroy",
-			  G_CALLBACK (cb_dialog_graphic_config_destroy), state);
-	g_signal_connect (G_OBJECT (glade_xml_get_widget (state->gui, "ok_button")),
+	g_signal_connect_swapped (G_OBJECT (glade_xml_get_widget (state->gui, "ok_button")),
 			  "clicked",
-			  G_CALLBACK (cb_dialog_graphic_config_ok_clicked), state);
+			  G_CALLBACK (gtk_widget_destroy), state->dialog);
 	g_signal_connect (G_OBJECT (glade_xml_get_widget (state->gui, "cancel_button")),
 			  "clicked",
 			  G_CALLBACK (cb_dialog_graphic_config_cancel_clicked), state);
+
+	/* a candidate for merging into attach guru */
 	gnumeric_init_help_button (
 		glade_xml_get_widget (state->gui, "help_button"),
-		(sog->type != SHEET_OBJECT_ARROW) ? "so-line.html" : "so-arrow.html");
-
+		(sog->type != SHEET_OBJECT_ARROW) ? GNUMERIC_HELP_LINK_SO_LINE
+		: GNUMERIC_HELP_LINK_SO_ARROW);
 	gnumeric_keyed_dialog (state->wbcg, GTK_WINDOW (state->dialog),
-			       SHEET_OBJECT_CONFIG_KEY);
-
+		SHEET_OBJECT_CONFIG_KEY);
+	g_object_set_data_full (G_OBJECT (state->dialog),
+		"state", state, (GDestroyNotify) cb_dialog_graphic_config_destroy);
+	gnumeric_non_modal_dialog (wbcg_toplevel (state->wbcg),
+		GTK_WINDOW (state->dialog));
 	wbcg_edit_attach_guru (state->wbcg, state->dialog);
-
+	gtk_widget_show (state->dialog);
 }
 
 static void
@@ -616,14 +603,14 @@ sheet_object_graphic_class_init (GObjectClass *object_class)
 	SheetObjectClass	*so_class  = SHEET_OBJECT_CLASS (object_class);
 	SheetObjectGraphicClass *sog_class = SHEET_OBJECT_GRAPHIC_CLASS (object_class);
 
-	sheet_object_graphic_parent_class = g_type_class_peek (SHEET_OBJECT_TYPE);
+	sheet_object_graphic_parent_class = g_type_class_peek_parent (object_class);
 
 	/* Object class method overrides */
 	object_class->finalize = sheet_object_graphic_finalize;
 
 	/* SheetObject class method overrides */
 	so_class->new_view	= sheet_object_graphic_new_view;
-	so_class->update_bounds = sheet_object_graphic_update_bounds;
+	so_class->update_view_bounds = sheet_object_graphic_update_bounds;
 	so_class->read_xml	= sheet_object_graphic_read_xml;
 	so_class->write_xml	= sheet_object_graphic_write_xml;
 	so_class->clone         = sheet_object_graphic_clone;
@@ -833,41 +820,38 @@ sheet_object_filled_clone (SheetObject const *so, Sheet *sheet)
 	return SHEET_OBJECT (new_sof);
 }
 
-typedef struct
-{
-	GladeXML           *gui;
-	GtkWidget          *dialog;
+typedef struct {
+	GladeXML	*gui;
+	GtkWidget	*dialog;
+	GtkSpinButton	*spin_border_width;
 
-	GtkWidget *fill_color_combo;
-	GtkWidget *outline_color_combo;
-	GtkSpinButton *spin_border_width;
-
-	StyleColor *outline_color;
-	StyleColor *fill_color;
-	double width;
+	StyleColor	*outline_color;
+	StyleColor	*fill_color;
+	double		 width;
 
 	WorkbookControlGUI *wbcg;
-	SheetObjectFilled *sof;
+	SheetObjectFilled  *sof;
 	Sheet		   *sheet;
 } DialogFilledData;
 
-static gboolean
-cb_dialog_filled_config_destroy (GtkObject *w, DialogFilledData *state)
+static void
+cb_dialog_filled_config_destroy (DialogFilledData *state)
 {
-	g_return_val_if_fail (w != NULL, FALSE);
-	g_return_val_if_fail (state != NULL, FALSE);
-
 	wbcg_edit_detach_guru (state->wbcg);
-
+	if (state->outline_color != NULL) {
+		style_color_unref (state->outline_color);
+		state->outline_color = NULL;
+	}
+	if (state->fill_color != NULL) {
+		style_color_unref (state->fill_color);
+		state->fill_color = NULL;
+	}
 	if (state->gui != NULL) {
 		g_object_unref (G_OBJECT (state->gui));
 		state->gui = NULL;
 	}
-
 	state->dialog = NULL;
 	g_free (state);
-
-	return FALSE;
 }
 
 static void
@@ -881,16 +865,21 @@ cb_dialog_filled_adjustment_value_changed (GtkAdjustment *adj, DialogFilledData
 }
 
 static void
-cb_dialog_filled_color_changed (ColorCombo *color_combo, GdkColor *color,
-		       gboolean is_custom, gboolean by_user, gboolean is_default,
-		       DialogFilledData *state)
+cb_fillcolor_changed (GtkWidget *cc, GdkColor *color,
+		      gboolean is_custom, gboolean by_user, gboolean is_default,
+		      SheetObject *so)
+{
+	sheet_object_graphic_fill_color_set (so,
+		color_combo_get_style_color (cc));
+}
+
+static void
+cb_outlinecolor_changed (GtkWidget *cc, GdkColor *color,
+			 gboolean is_custom, gboolean by_user, gboolean is_default,
+			 SheetObject *so)
 {
-	SheetObject *so = SHEET_OBJECT (state->sof);
-	sheet_object_graphic_fill_color_set (so, color_combo_get_style_color (
-						     state->fill_color_combo));
 	sheet_object_filled_outline_color_set (so,
-					       color_combo_get_style_color (
-						       state->outline_color_combo));
+		color_combo_get_style_color (cc));
 }
 
 static void
@@ -907,9 +896,11 @@ cb_dialog_filled_config_cancel_clicked (GtkWidget *button, DialogFilledData *sta
 
 	sheet_object_graphic_width_set (sog, state->width);
 	sheet_object_graphic_fill_color_set (so,
-					     state->fill_color);
+		state->fill_color);
+	state->fill_color = NULL;
 	sheet_object_filled_outline_color_set (so,
-					       state->outline_color);
+		state->outline_color);
+	state->outline_color = NULL;
 
 	gtk_widget_destroy (state->dialog);
 }
@@ -920,8 +911,8 @@ sheet_object_filled_user_config (SheetObject *so, SheetControl *sc)
 	SheetObjectFilled *sof = SHEET_OBJECT_FILLED (so);
 	SheetObjectGraphic *sog = SHEET_OBJECT_GRAPHIC (so);
 	WorkbookControlGUI *wbcg = scg_get_wbcg (SHEET_CONTROL_GUI (sc));
-	GtkWidget *table;
 	DialogFilledData *state;
+	GtkWidget *table, *w;
 
 	g_return_if_fail (IS_SHEET_OBJECT_FILLED (so));
 
@@ -934,69 +925,73 @@ sheet_object_filled_user_config (SheetObject *so, SheetControl *sc)
 	state->wbcg = wbcg;
 	state->sheet = sc_sheet	(sc);
 
-	state->gui = gnumeric_glade_xml_new (wbcg, "so-fill.glade");
+	state->gui = gnm_glade_xml_new (COMMAND_CONTEXT (wbcg),
+		"so-fill.glade", NULL, NULL);
 	state->dialog = glade_xml_get_widget (state->gui, "SO-Fill");
 
  	table = glade_xml_get_widget (state->gui, "table");
 
-	state->outline_color_combo = color_combo_new (NULL, _("Transparent"),
-				NULL, color_group_fetch ("outline_color", so));
-	color_combo_set_color (COLOR_COMBO (state->outline_color_combo),
-			       sof->outline_color ? &sof->outline_color->color : NULL);
+	w = color_combo_new (NULL, _("Transparent"),
+		NULL, color_group_fetch ("outline_color", so));
+	gtk_label_set_mnemonic_widget (
+		GTK_LABEL (glade_xml_get_widget (state->gui, "border_label")), w);
+	color_combo_set_color (COLOR_COMBO (w),
+		sof->outline_color ? &sof->outline_color->color : NULL);
+	color_combo_set_instant_apply (COLOR_COMBO (w), FALSE);
+	gtk_combo_box_set_tearable (GTK_COMBO_BOX (w), FALSE);
 	state->outline_color = style_color_ref (sof->outline_color);
-	gtk_table_attach_defaults (GTK_TABLE (table),
-				   state->outline_color_combo, 1, 2, 0, 1);
-	gtk_combo_box_set_arrow_relief (GTK_COMBO_BOX (state->outline_color_combo), 
-					GTK_RELIEF_NORMAL);
-	color_combo_box_set_preview_relief (COLOR_COMBO (state->outline_color_combo), 
-					    GTK_RELIEF_NORMAL);
-	gtk_widget_show (GTK_WIDGET (state->outline_color_combo));
+	gtk_table_attach_defaults (GTK_TABLE (table), w, 1, 2, 0, 1);
+	gtk_combo_box_set_arrow_relief (GTK_COMBO_BOX (w), GTK_RELIEF_NORMAL);
+	color_combo_box_set_preview_relief (COLOR_COMBO (w), GTK_RELIEF_NORMAL);
+	gtk_widget_show (GTK_WIDGET (w));
+	g_signal_connect (G_OBJECT (w),
+		"color_changed",
+		G_CALLBACK (cb_outlinecolor_changed), so);
 
-
-	state->fill_color_combo = color_combo_new (NULL, _("Transparent"),
-				NULL, color_group_fetch ("fill_color", so));
-	color_combo_set_color (COLOR_COMBO (state->fill_color_combo),
-			       sog->fill_color ? &sog->fill_color->color : NULL);
+	w = color_combo_new (NULL, _("Transparent"),
+		NULL, color_group_fetch ("fill_color", so));
+	gtk_label_set_mnemonic_widget (
+		GTK_LABEL (glade_xml_get_widget (state->gui, "fill_label")), w);
+	color_combo_set_color (COLOR_COMBO (w),
+		sog->fill_color ? &sog->fill_color->color : NULL);
+	color_combo_set_instant_apply (COLOR_COMBO (w), FALSE);
+	gtk_combo_box_set_tearable (GTK_COMBO_BOX (w), FALSE);
 	state->fill_color = style_color_ref (sog->fill_color);
-	gtk_table_attach_defaults (GTK_TABLE (table),
-				   state->fill_color_combo, 1, 2, 1, 2);
-	gtk_combo_box_set_arrow_relief (GTK_COMBO_BOX (state->fill_color_combo), 
-					GTK_RELIEF_NORMAL);
-	color_combo_box_set_preview_relief (COLOR_COMBO (state->fill_color_combo), 
-					    GTK_RELIEF_NORMAL);
-	gtk_widget_show (GTK_WIDGET (state->fill_color_combo));
+	gtk_table_attach_defaults (GTK_TABLE (table), w, 1, 2, 1, 2);
+	gtk_combo_box_set_arrow_relief (GTK_COMBO_BOX (w), GTK_RELIEF_NORMAL);
+	color_combo_box_set_preview_relief (COLOR_COMBO (w), GTK_RELIEF_NORMAL);
+	gtk_widget_show (GTK_WIDGET (w));
+	g_signal_connect (G_OBJECT (w),
+		"color_changed",
+		G_CALLBACK (cb_fillcolor_changed), so);
 
 	state->spin_border_width = GTK_SPIN_BUTTON (glade_xml_get_widget (
 						     state->gui, "spin_border_width"));
 	state->width = sog->width;
 	gtk_spin_button_set_value (state->spin_border_width, state->width);
+	gnumeric_editable_enters (GTK_WINDOW (state->dialog),
+				  GTK_WIDGET (state->spin_border_width));
 
-
-	g_signal_connect (G_OBJECT (state->fill_color_combo),
-			  "color_changed",
-			  G_CALLBACK (cb_dialog_filled_color_changed), state);
-	g_signal_connect (G_OBJECT (state->outline_color_combo),
-			  "color_changed",
-			  G_CALLBACK (cb_dialog_filled_color_changed), state);
 	g_signal_connect (G_OBJECT
 			  (gtk_spin_button_get_adjustment (state->spin_border_width)),
 			  "value_changed",
 			  G_CALLBACK (cb_dialog_filled_adjustment_value_changed), state);
-	g_signal_connect (G_OBJECT (state->dialog),
-			  "destroy",
-			  G_CALLBACK (cb_dialog_filled_config_destroy), state);
 	g_signal_connect (G_OBJECT (glade_xml_get_widget (state->gui, "ok_button")),
 			  "clicked",
 			  G_CALLBACK (cb_dialog_filled_config_ok_clicked), state);
 	g_signal_connect (G_OBJECT (glade_xml_get_widget (state->gui, "cancel_button")),
 			  "clicked",
 			  G_CALLBACK (cb_dialog_filled_config_cancel_clicked), state);
+
+	/* a candidate for merging into attach guru */
 	gnumeric_init_help_button (glade_xml_get_widget (state->gui, "help_button"),
-				   "so-filled.html");
-
+		GNUMERIC_HELP_LINK_SO_FILLED);
 	gnumeric_keyed_dialog (state->wbcg, GTK_WINDOW (state->dialog),
-			       SHEET_OBJECT_CONFIG_KEY);
-
+		SHEET_OBJECT_CONFIG_KEY);
+	g_object_set_data_full (G_OBJECT (state->dialog),
+		"state", state, (GDestroyNotify) cb_dialog_filled_config_destroy);
+	gnumeric_non_modal_dialog (wbcg_toplevel (state->wbcg),
+		GTK_WINDOW (state->dialog));
 	wbcg_edit_attach_guru (state->wbcg, state->dialog);
 	gtk_widget_show (state->dialog);
 }
@@ -1084,57 +1079,35 @@ make_ellipse (GnomePrintContext *ctx,
 
 static void
 sheet_object_filled_print (SheetObject const *so, GnomePrintContext *ctx,
-			   double base_x, double base_y)
+			   double width, double height)
 {
-	SheetObjectFilled *sof;
-	SheetObjectGraphic *sog;
-	double coords [4];
-	double start_x, start_y;
-	double end_x, end_y;
+	SheetObjectFilled  *sof = SHEET_OBJECT_FILLED (so);
+	SheetObjectGraphic *sog = SHEET_OBJECT_GRAPHIC (so);
 
-	g_return_if_fail (IS_SHEET_OBJECT_FILLED (so));
-	g_return_if_fail (GNOME_IS_PRINT_CONTEXT (ctx));
-	sof = SHEET_OBJECT_FILLED (so);
-	sog = SHEET_OBJECT_GRAPHIC (so);
-
-	sheet_object_position_pts_get (so, coords);
-
-	start_x = base_x;
-	start_y = base_y;
-	end_x = start_x + (coords [2] - coords [0]);
-	end_y = start_y + (coords [3] - coords [1]);
-
-	gnome_print_gsave (ctx);
+	gnome_print_newpath (ctx);
+	if (sog->type == SHEET_OBJECT_OVAL)
+		make_ellipse (ctx, 0., width, 0., -height);
+	else
+		make_rect (ctx, 0., width, 0., -height);
+	gnome_print_closepath (ctx);
 
+	if (sog->fill_color) {
+		gnome_print_gsave (ctx);
+		gnome_print_setrgbcolor (ctx,
+			sog->fill_color->color.red   / (double) 0xffff,
+			sog->fill_color->color.green / (double) 0xffff,
+			sog->fill_color->color.blue  / (double) 0xffff);
+		gnome_print_fill (ctx);
+		gnome_print_grestore (ctx);
+	}
 	if (sof->outline_color) {
 		gnome_print_setlinewidth (ctx, sog->width);
 		gnome_print_setrgbcolor (ctx,
-			sof->outline_color->red   / (double) 0xffff,
-			sof->outline_color->green / (double) 0xffff,
-			sof->outline_color->blue  / (double) 0xffff);
-		gnome_print_newpath (ctx);
-		if (sog->type == SHEET_OBJECT_OVAL)
-			make_ellipse (ctx, start_x, end_x, start_y, end_y);
-		else
-			make_rect (ctx, start_x, end_x, start_y, end_y);
-		gnome_print_closepath (ctx);
+			sof->outline_color->color.red   / (double) 0xffff,
+			sof->outline_color->color.green / (double) 0xffff,
+			sof->outline_color->color.blue  / (double) 0xffff);
 		gnome_print_stroke (ctx);
 	}
-
-	if (sog->fill_color) {
-		gnome_print_setrgbcolor (ctx,
-			sog->fill_color->red   / (double) 0xffff,
-			sog->fill_color->green / (double) 0xffff,
-			sog->fill_color->blue  / (double) 0xffff);
-		gnome_print_newpath (ctx);
-		if (sog->type == SHEET_OBJECT_OVAL)
-			make_ellipse (ctx, start_x, end_x, start_y, end_y);
-		else
-			make_rect (ctx, start_x, end_x, start_y, end_y);
-		gnome_print_fill (ctx);
-	}
-
-	gnome_print_grestore (ctx);
 }
 
 static void
@@ -1142,13 +1115,13 @@ sheet_object_filled_class_init (GObjectClass *object_class)
 {
 	SheetObjectClass *sheet_object_class;
 
-	sheet_object_filled_parent_class = g_type_class_peek (SHEET_OBJECT_GRAPHIC_TYPE);
+	sheet_object_filled_parent_class = g_type_class_peek_parent (object_class);
 
 	object_class->finalize		  = sheet_object_filled_finalize;
 
 	sheet_object_class = SHEET_OBJECT_CLASS (object_class);
 	sheet_object_class->new_view	  = sheet_object_filled_new_view;
-	sheet_object_class->update_bounds = sheet_object_filled_update_bounds;
+	sheet_object_class->update_view_bounds = sheet_object_filled_update_bounds;
 	sheet_object_class->read_xml	  = sheet_object_filled_read_xml;
 	sheet_object_class->write_xml	  = sheet_object_filled_write_xml;
 	sheet_object_class->clone         = sheet_object_filled_clone;
@@ -1316,7 +1289,7 @@ sheet_object_polygon_class_init (GObjectClass *object_class)
 {
 	SheetObjectClass *sheet_object_class;
 
-	sheet_object_polygon_parent_class = g_type_class_peek (SHEET_OBJECT_TYPE);
+	sheet_object_polygon_parent_class = g_type_class_peek_parent (object_class);
 
 	/* Object class method overrides */
 	object_class->finalize = sheet_object_polygon_finalize;
@@ -1324,7 +1297,7 @@ sheet_object_polygon_class_init (GObjectClass *object_class)
 	/* SheetObject class method overrides */
 	sheet_object_class = SHEET_OBJECT_CLASS (object_class);
 	sheet_object_class->new_view	  = sheet_object_polygon_new_view;
-	sheet_object_class->update_bounds = sheet_object_polygon_update_bounds;
+	sheet_object_class->update_view_bounds = sheet_object_polygon_update_bounds;
 	sheet_object_class->read_xml	  = sheet_object_polygon_read_xml;
 	sheet_object_class->write_xml	  = sheet_object_polygon_write_xml;
 	sheet_object_class->clone         = sheet_object_polygon_clone;
@@ -1520,6 +1493,7 @@ sheet_object_text_update_bounds (SheetObject *so, GObject *view)
 
 	foo_canvas_item_set (FOO_CANVAS_ITEM (group->item_list->next->data),
 		"clip_width",  fabs (coords [0] - coords [2]),
+		"wrap_width",  fabs (coords [0] - coords [2]),
 		"clip_height", fabs (coords [1] - coords [3]),
 		NULL);
 	foo_canvas_item_set (FOO_CANVAS_ITEM (group->item_list->data),
@@ -1599,7 +1573,7 @@ sheet_object_text_class_init (GObjectClass *object_class)
 	SheetObjectClass *so_class = SHEET_OBJECT_CLASS (object_class);
 	SheetObjectGraphicClass *sog_class = SHEET_OBJECT_GRAPHIC_CLASS (object_class);
 
-	sheet_object_text_parent_class = g_type_class_peek (SHEET_OBJECT_FILLED_TYPE);
+	sheet_object_text_parent_class = g_type_class_peek_parent (object_class);
 
 	/* Object class method overrides */
 	object_class->finalize = sheet_object_text_finalize;
@@ -1607,11 +1581,11 @@ sheet_object_text_class_init (GObjectClass *object_class)
 	/* SheetObject class method overrides */
 	so_class = SHEET_OBJECT_CLASS (object_class);
 	so_class->new_view	= sheet_object_text_new_view;
-	so_class->update_bounds = sheet_object_text_update_bounds;
+	so_class->update_view_bounds = sheet_object_text_update_bounds;
 	so_class->read_xml	= sheet_object_text_read_xml;
 	so_class->write_xml	= sheet_object_text_write_xml;
 	so_class->clone         = sheet_object_text_clone;
-	/* so_class->user_config   = NULL; inherit from parent */ 
+	/* so_class->user_config   = NULL; inherit from parent */
 	/* so_class->print         = NULL; inherit from parent */
 	so_class->rubber_band_directly = FALSE;
 
