@@ -27,6 +27,10 @@
 #include "sheet-object-graphic.h"
 #include "sheet-object-cell-comment.h"
 #include "sheet-object-widget.h"
+#include "sheet-object-graph.h"
+#include "sheet-object-image.h"
+#include "graph.h"
+#include "io-context.h"
 
 #include <libxml/globals.h>
 #include <gtk/gtkimagemenuitem.h>
@@ -276,7 +280,6 @@ sheet_object_class_init (GObjectClass *klass)
 	sheet_object_class->populate_menu        = sheet_object_populate_menu;
 	sheet_object_class->print                = NULL;
 	sheet_object_class->user_config          = NULL;
-	sheet_object_class->stipple_border	 = FALSE;
 	sheet_object_class->rubber_band_directly = FALSE;
 	sheet_object_class->default_size	 = so_default_size;
 }
@@ -493,10 +496,10 @@ sheet_object_new_view (SheetObject *so, SheetControl *sc, gpointer key)
 
 void
 sheet_object_print (SheetObject const *so, GnomePrintContext *ctx,
-		    double base_x, double base_y)
+		    double width, double height)
 {
 	if (SO_CLASS (so)->print)
-		SO_CLASS (so)->print (so, ctx, base_x, base_y);
+		SO_CLASS (so)->print (so, ctx, width, height);
 }
 
 SheetObject *
@@ -516,10 +519,24 @@ sheet_object_read_xml (XmlParseContext const *ctxt, xmlNodePtr tree)
 		so = sheet_object_line_new (TRUE);
 	} else if (!strcmp (tree->name, "Line")){
 		so = sheet_object_line_new (FALSE);
+
+	/* Graphs changed names */
+	} else if (!strcmp (tree->name, "GnmGraph")){
+		so = sheet_object_graph_new (FALSE);
+
 	} else {
-		obj = g_object_new (g_type_from_name ((gchar *)tree->name), NULL);
-		if (!obj)
-			return (NULL);
+		GType type = g_type_from_name ((gchar *)tree->name);
+		if (type == 0) {
+			char *str = g_strdup_printf (_("Unsupported object type '%s'"),
+						     tree->name);
+			gnm_io_warning_unsupported_feature (ctxt->io_context, str);
+			g_free (str);
+			return NULL;
+		}
+
+		obj = g_object_new (type, NULL);
+		if (obj == NULL)
+			return NULL;
 
 		so = SHEET_OBJECT (obj);
 	}
@@ -888,7 +905,6 @@ sheet_objects_clear (Sheet const *sheet, Range const *r, GType t)
  * they introduce automake depends in the non-bonobo build.
  */
 extern GType sheet_object_bonobo_get_type (void);
-extern GType gnm_graph_get_type (void);
 #endif
 
 void
@@ -896,12 +912,15 @@ sheet_object_register (void)
 {
 	SHEET_OBJECT_GRAPHIC_TYPE;
 	SHEET_OBJECT_FILLED_TYPE;
+	SHEET_OBJECT_TEXT_TYPE;
+	SHEET_OBJECT_GRAPH_TYPE;
+	SHEET_OBJECT_IMAGE_TYPE;
+	GNM_GO_DATA_SCALAR_TYPE;
+	GNM_GO_DATA_VECTOR_TYPE;
 	CELL_COMMENT_TYPE;
-	(void) sheet_object_text_get_type ();
 #ifdef WITH_BONOBO
 #ifdef GNOME2_CONVERSION_COMPLETE
 	sheet_object_bonobo_get_type ();
-	gnm_graph_get_type ();
 #endif
 #endif
 	sheet_object_widget_register ();
@@ -929,6 +948,7 @@ sheet_object_clone (SheetObject const *so, Sheet *sheet)
 	new_so->type = so->type;
 	sheet_object_anchor_cpy (&new_so->anchor, &so->anchor);
 	sheet_object_set_sheet (new_so, sheet);
+	g_object_unref (new_so);
 
 	return new_so;
 }
