@@ -83,8 +83,12 @@ sv_wbv (SheetView const *sv)
 static void
 sv_init_sc (SheetView const *sv, SheetControl *sc)
 {
+	CellPos initial;
+
+	sc_set_zoom_factor (sc);
+
 	/* set_panes will change the initial so cache it */
-	CellPos initial = sv->initial_top_left;
+	initial = sv->initial_top_left;
 	sc_set_panes (sc);
 
 	/* And this will restore it */
@@ -554,8 +558,7 @@ sv_update (SheetView *sv)
 			if (new_pos == NULL)
 				new_pos = cellpos_as_string (&sv->edit_pos);
 			SHEET_VIEW_FOREACH_CONTROL (sv, sc,
-				wb_control_selection_descr_set (sc_wbc (sc), new_pos);
-				wb_control_menu_state_update (sc_wbc (sc), MS_ADD_VS_REMOVE_FILTER););
+				wb_control_selection_descr_set (sc_wbc (sc), new_pos););
 		}
 	}
 
@@ -567,6 +570,8 @@ sv_update (SheetView *sv)
 			sv->auto_expr_timer = g_timeout_add_full (0, abs (lag), /* seems ok */
 				cb_update_auto_expr, (gpointer) sv, NULL);
 		}
+		SHEET_VIEW_FOREACH_CONTROL (sv, sc,
+			wb_control_menu_state_update (sc_wbc (sc), MS_ADD_VS_REMOVE_FILTER););
 	}
 }
 
@@ -697,7 +702,7 @@ sv_panes_insdel_colrow (SheetView *sv, gboolean is_cols,
 	CellPos br = sv->unfrozen_top_left;
 
 	if (is_cols) {
-		/* unfrozen or acting in unfrozen region */
+		/* ignore if not frozen, or acting in unfrozen region */
 		if (br.col <= tl.col || br.col <= start)
 			return;
 		if (is_insert) {
@@ -705,9 +710,12 @@ sv_panes_insdel_colrow (SheetView *sv, gboolean is_cols,
 			if (tl.col >= start)	/* UI can not do this */
 				tl.col += count;
 		} else {
+			br.col -= count;
+			if (br.col <= tl.col)
+				br.col = tl.col + 1;
 		}
 	} else {
-		/* unfrozen or acting in unfrozen region */
+		/* ignore if not frozen, or acting in unfrozen region */
 		if (br.row <= tl.row || br.row <= start)
 			return;
 		if (is_insert) {
@@ -715,6 +723,9 @@ sv_panes_insdel_colrow (SheetView *sv, gboolean is_cols,
 			if (tl.row >= start)	/* UI can not do this */
 				tl.row += count;
 		} else {
+			br.row -= count;
+			if (br.row <= tl.row)
+				br.row = tl.row + 1;
 		}
 	}
 	sv_freeze_panes (sv, &tl, &br);
