@@ -32,7 +32,6 @@
 #include "sheet-style.h"
 #include "style.h"
 #include "mstyle.h"
-#include "formats.h"
 #include "io-context-priv.h"
 #include "command-context.h"
 #include "workbook-control.h"
@@ -46,14 +45,17 @@
 #include "format.h"
 #include "selection.h"
 #include "ranges.h"
+#include "clipboard.h"
 #include "parse-util.h"
+#include "commands.h"
 
 #include <gsf/gsf-input.h>
-#include <ctype.h>
 #include <string.h>
 #include <glade/glade.h>
 #include <gsf/gsf-output.h>
 #include <gsf/gsf-output-memory.h>
+#include <gsf/gsf-utils.h>
+#include <locale.h>
 
 /**
  * stf_open_and_read
@@ -86,25 +88,24 @@ stf_open_and_read (GsfInput *input, size_t *readsize)
 	if (result == NULL)
 		return NULL;
 
-	if (gsf_input_read (input, *readsize, result) == NULL) {
+	*((char *)result + *readsize) = '\0';
+
+	if (*readsize > 0 && gsf_input_read (input, *readsize, result) == NULL) {
+		g_warning ("gsf_input_read failed.");
 		g_free (result);
 		result = NULL;
 	}
 
-	*((char *)result + *readsize) = '\0';
-
 	return result;
 }
 
 static char *
-stf_preparse (CommandContext *context, GsfInput *input, gchar const *enc)
+stf_preparse (CommandContext *context, GsfInput *input, size_t *data_len)
 {
 	char *data;
-/*	unsigned char const *c;*/
-	size_t len;
-	
-	data = stf_open_and_read (input, &len);
-	
+
+	data = stf_open_and_read (input, data_len);
+
 	if (!data) {
 		if (context)
 			gnumeric_error_read (context,
@@ -112,126 +113,48 @@ stf_preparse (CommandContext *context, GsfInput *input, gchar const *enc)
 		return NULL;
 	}
 
-#if 0
-	/* This would only make sense for some encodings */
-	len = stf_parse_convert_to_unix (data);
-	if (len < 0) {
-		g_free (data);
-		if (context)
-			gnumeric_error_read (context,
-				_("Error while trying to pre-convert file"));
-		return NULL;
-	}
-
-#endif
-
-        /* Now that the data is read we have to translate it into the   */
-	/* utf8 encoding. I tis impossible to do anything with the file */
-        /* unless we know the encoding it is in.                        */
-
-        if (enc == NULL)
-		        g_get_charset (&enc);
-	
-	{
-		char *result;
-		gsize bytes_read = -1;
-		gsize bytes_written = -1;
-		GError *error = NULL;
-
-		/* FIXME: check for overflow in buf_len conversion */
-		result = g_convert_with_fallback (data, len, "UTF-8", enc, NULL,
-						  &bytes_read, &bytes_written, &error);
-		if (error) {
-			char *msg = NULL;
-			if (bytes_read < len) {
-				msg = g_strdup_printf (_("The file does not appear to use the %s encoding.\n"
-                                                         "After %i of %i bytes a conversion error occurred:\n%s.\n"
-							 "The next two bytes have (hex) values %02X and %02X."),
-						       enc, bytes_read, len, error->message, 
-						       (unsigned int)*((unsigned char *)(data + bytes_read)),
-						       (unsigned int)*((unsigned char *)(data + bytes_read + 1)));
-			} else {
-				msg = g_strdup_printf (_("The file does not appear to use the %s encoding.\n"
-							 "After %i of %i bytes a conversion error occurred:\n%s.\n"), 
-						       enc, bytes_read, len, error->message);
-			}
-			if (context)
-				gnumeric_error_read (context, msg);
-			g_warning (msg);
-			g_error_free (error);
-			g_free (msg);
-		}
-		
-		g_free (data);
-		data = result;
-	}
-	
-
-#if 0
-
-	if ((c = stf_parse_is_valid_data (data, len)) != NULL) {
-		if (context) {
-			char *msg;
-			char *invalid_char = g_locale_to_utf8 (c, 1, NULL, NULL, NULL);
-
-			/* if locale conversion failed try 8859-1 as a fallback
-			 * it is ok to do this for an error message */
-			if (invalid_char == NULL)
-				invalid_char = g_convert (c, 1, "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
-			if (invalid_char != NULL) {
-				msg = g_strdup_printf (_("This file does not seem to be a valid text file.\n"
-							 "The character '%s' (ASCII decimal %d) was encountered.\n"
-							 "Most likely your locale settings are wrong."),
-						       invalid_char, (int)*c);
-				g_free (invalid_char);
-			} else
-				msg = g_strdup_printf (_("This file does not seem to be a valid text file.\n"
-							 "Byte 0x%d was encountered.\n"
-							 "Most likely your locale settings are wrong."),
-						       (int)*c);
-			gnumeric_error_read (context, msg);
-			g_free (msg);
-		}
-		g_free (data);
-		return NULL;
-	}
-#endif
-
 	return data;
 }
 
+static void
+stf_apply_formats (StfParseOptions_t *parseoptions,
+		   Sheet *sheet, int col, int start_row, int end_row)
+{
+	unsigned int ui;
+	Range range;
+
+	/* If we didn't use the stf dialog, then formats will be NULL */
+	if (parseoptions->formats == NULL)
+		return;
+
+	range.start.col = col;
+	range.start.row = start_row;
+	range.end.col   = col;
+	range.end.row   = end_row;
+
+	for (ui = 0; ui < parseoptions->formats->len; ui++) {
+		if (parseoptions->col_import_array == NULL ||
+		    parseoptions->col_import_array[ui]) {
+			MStyle *style = mstyle_new ();
+			StyleFormat *sf = g_ptr_array_index 
+				(parseoptions->formats, ui);
+			mstyle_set_format (style, sf);
+			sheet_style_apply_range (sheet, &range, style);
+			range.start.col++;
+			range.end.col++;
+		}
+	}
+}
+
 static gboolean
 stf_store_results (DialogStfResult_t *dialogresult,
 		   Sheet *sheet, int start_col, int start_row)
 {
-	GSList *iterator;
-	int col, rowcount;
-
-	stf_parse_options_set_lines_to_parse (dialogresult->parseoptions, dialogresult->lines);
-
-	iterator = dialogresult->formats;
-	col = start_col;
-	rowcount = stf_parse_get_rowcount (dialogresult->parseoptions, dialogresult->newstart);
-	while (iterator) {
-		Range range;
-		MStyle *style = mstyle_new ();
-
-		mstyle_set_format (style, iterator->data);
-
-		range.start.col = col;
-		range.start.row = start_row;
-		range.end.col   = col;
-		range.end.row   = rowcount;
-
-		sheet_style_apply_range (sheet, &range, style);
-
-		iterator = g_slist_next (iterator);
-
-		col++;
-	}
-
+	stf_apply_formats (dialogresult->parseoptions, 
+			   sheet, start_col, start_row, 
+			   start_row + dialogresult->rowcount - 1);
 	return stf_parse_sheet (dialogresult->parseoptions,
-				dialogresult->newstart, sheet,
+				dialogresult->text, NULL, sheet,
 				start_col, start_row);
 }
 
@@ -246,38 +169,53 @@ stf_store_results (DialogStfResult_t *dialogresult,
  * Main routine, handles importing a file including all dialog mumbo-jumbo
  **/
 static void
-stf_read_workbook (GnmFileOpener const *fo,  gchar const *enc, 
+stf_read_workbook (GnmFileOpener const *fo,  gchar const *enc,
 		   IOContext *context, WorkbookView *wbv, GsfInput *input)
 {
 	DialogStfResult_t *dialogresult = NULL;
-	char *name;
+	char *name, *nameutf8;
 	char *data;
+	size_t data_len;
 	Sheet *sheet;
 	Workbook *book;
 
-	data = stf_preparse (COMMAND_CONTEXT (context), input, enc);
-	if (!data)
-		return;
-
 	/* FIXME : how to do this cleanly ? */
 	if (!IS_WORKBOOK_CONTROL_GUI (context->impl))
 		return;
 
-	/* Add Sheet */
 	name = g_path_get_basename (gsf_input_name (input));
+	nameutf8 = g_filename_to_utf8 (name, -1, NULL, NULL, NULL);
+	g_free (name);
+	if (!nameutf8) {
+		g_warning ("Failed to convert filename to UTF-8.  This shouldn't happen here.");
+		return;
+	}
+
+	data = stf_preparse (COMMAND_CONTEXT (context), input, &data_len);
+	if (!data) {
+		g_free (nameutf8);
+		return;
+	}
+
+	/* Add Sheet */
 	book = wb_view_workbook (wbv);
-	sheet = sheet_new (book, name);
+	sheet = sheet_new (book, nameutf8);
 	workbook_sheet_attach (book, sheet, NULL);
 
-	dialogresult = stf_dialog (WORKBOOK_CONTROL_GUI (context->impl), name, data);
-	g_free (name);
+	dialogresult = stf_dialog (WORKBOOK_CONTROL_GUI (context->impl),
+				   enc, FALSE, NULL, FALSE, 
+				   nameutf8, data, data_len);
 	if (dialogresult != NULL && stf_store_results (dialogresult, sheet, 0, 0)) {
 		workbook_recalc (book);
 		sheet_queue_respan (sheet, 0, SHEET_MAX_ROWS-1);
-	} else
-		workbook_sheet_detach (book, sheet);
+	} else {
+		/* the user has cancelled */
+                /* the caller should notice that we have no sheets */
+		workbook_sheet_detach (book, sheet, TRUE);
+	}
 
 	g_free (data);
+	g_free (nameutf8);
 	if (dialogresult != NULL)
 		stf_dialog_result_free (dialogresult);
 }
@@ -321,6 +259,7 @@ stf_text_to_columns (WorkbookControl *wbc, CommandContext *cc)
 	Range		 target;
 	GsfOutputMemory	*buf;
 	guint8 const	*data;
+	size_t data_len;
 
 	sv    = wb_control_cur_sheet_view (wbc);
 	src_sheet = sv_sheet (sv);
@@ -350,35 +289,42 @@ stf_text_to_columns (WorkbookControl *wbc, CommandContext *cc)
 		src->end.col, src->end.row,
 		(CellIterFunc) &cb_get_content, buf);
 
-	gsf_output_write (GSF_OUTPUT (buf), 1, "\0");
 	gsf_output_close (GSF_OUTPUT (buf));
 	data = gsf_output_memory_get_bytes (buf);
-	dialogresult = stf_dialog (WORKBOOK_CONTROL_GUI (wbc),
-		_("Text to Columns"), data);
-
-	if (dialogresult == NULL ||
-	    !stf_store_results (dialogresult, target_sheet,
-				target.start.col, target.start.row)) {
+	data_len = (size_t)gsf_output_size (GSF_OUTPUT (buf));
+	if (data_len == 0) {
 		gnumeric_error_read (COMMAND_CONTEXT (cc),
-			_("Error while trying to parse data into sheet"));
+					     _("There is no data "
+					       "to convert"));
 	} else {
-		sheet_flag_status_update_range (target_sheet, &target);
-		sheet_queue_respan (target_sheet, target.start.row, target.end.row);
-		workbook_recalc (target_sheet->workbook);
-		sheet_redraw_all (target_sheet, FALSE);
+		dialogresult = stf_dialog (WORKBOOK_CONTROL_GUI (wbc),
+					   NULL, FALSE, NULL, FALSE,
+					   _("Text to Columns"), 
+					   data, data_len);
+	}
+	if (dialogresult != NULL) {
+		CellRegion *cr = NULL;
+
+		cr = stf_parse_region (dialogresult->parseoptions, dialogresult->text, NULL);
+		if (cr != NULL) {
+			stf_dialog_result_attach_formats_to_cr (dialogresult, cr);
+			target.end.col = target.start.col + cr->cols - 1;
+		}
+		if (cr == NULL || cmd_text_to_columns (wbc, src, src_sheet, 
+						      &target, target_sheet, cr))
+			gnumeric_error_read (COMMAND_CONTEXT (cc),
+					     _("Error while trying to "
+					       "parse data into sheet"));
+		stf_dialog_result_free (dialogresult);
 	}
 
 	g_object_unref (G_OBJECT (buf));
-	if (dialogresult != NULL)
-		stf_dialog_result_free (dialogresult);
 }
 
-#define STF_PROBE_SIZE 16384
-
 /**
  * stf_read_workbook_auto_csvtab
  * @fo       : file opener
- * @enc      : optional encoding 
+ * @enc      : optional encoding
  * @context  : command context
  * @book     : workbook
  * @input    : file to read from+convert
@@ -386,103 +332,52 @@ stf_text_to_columns (WorkbookControl *wbc, CommandContext *cc)
  * Attempt to auto-detect CSV or tab-delimited file
  **/
 static void
-stf_read_workbook_auto_csvtab (GnmFileOpener const *fo, gchar const *enc, 
+stf_read_workbook_auto_csvtab (GnmFileOpener const *fo, gchar const *enc,
 			       IOContext *context,
 			       WorkbookView *wbv, GsfInput *input)
 {
 	Sheet *sheet;
 	Workbook *book;
 	char *name;
-	char *data;
+	char *data, *utf8data;
+	size_t data_len;
 	StfParseOptions_t *po;
-	char *pos;
-	unsigned int sep = 0, tab = 0, lines = 0;
-	int i;
-	gboolean last_was_newline = FALSE;
 
-        gunichar guni_tab = '\t';
-	gunichar guni_newline = '\n';
-	gunichar guni_carriage = '\r';
-	gunichar guni_quote = '\"';
-	gunichar guni_sep = format_get_arg_sep ();
-	
+	g_return_if_fail (context != NULL);
+	g_return_if_fail (wbv != NULL);
+
 	book = wb_view_workbook (wbv);
-	data = stf_preparse (COMMAND_CONTEXT (context), input, enc);
+	data = stf_preparse (COMMAND_CONTEXT (context), input, &data_len);
 	if (!data)
 		return;
 
-        for (i = STF_PROBE_SIZE, pos = data ; pos && *pos && i-- > 0; 
-	     pos = stf_parse_next_token (pos, guni_quote, TRUE, NULL)) {
-		gunichar this_char;
-		
-		this_char = g_utf8_get_char (pos);
-		if (this_char == guni_sep) {
-			++sep; 
-			last_was_newline = FALSE;
-		} else if (this_char == guni_tab) {
-			++tab;
-			last_was_newline = FALSE;
-		} else if ((this_char == guni_newline || this_char == guni_carriage) 
-			   && !last_was_newline) {
-			++lines;
-			last_was_newline = TRUE;
-		}
+	enc = gnm_guess_encoding (data, data_len, enc, &utf8data);
+	g_free (data);
+
+	if (!enc) {
+		gnumeric_error_read (COMMAND_CONTEXT (context),
+				     _("That file is not in the given encoding."));
+		return;
 	}
 
+        po = stf_parse_options_guess (utf8data);
+
 	name = g_path_get_basename (gsf_input_name (input));
 	sheet = sheet_new (book, name);
-
+	g_free (name);
 	workbook_sheet_attach (book, sheet, NULL);
 
-	po = stf_parse_options_new ();
-
-	stf_parse_options_set_type (po, PARSE_TYPE_CSV);
-	stf_parse_options_set_trim_spaces (po, TRIM_TYPE_LEFT | TRIM_TYPE_RIGHT);
-	stf_parse_options_set_lines_to_parse (po, -1);
-
-	stf_parse_options_csv_set_stringindicator (po, '"');
-	stf_parse_options_csv_set_indicator_2x_is_single (po, TRUE);
-	stf_parse_options_csv_set_duplicates (po, FALSE);
-
-	/* Guess */
-	stf_parse_options_csv_set_separators (po, (guni_sep == ',') ? ",":";", NULL);
-	if (tab > lines || tab >= sep)
-		stf_parse_options_csv_set_separators (po, "\t", NULL);
-
-	if (!stf_parse_sheet (po, data, sheet, 0, 0)) {
-
-		workbook_sheet_detach (book, sheet);
-		g_free (data);
-		stf_parse_options_free (po);
+	if (stf_parse_sheet (po, utf8data, NULL, sheet, 0, 0)) {
+		workbook_recalc (book);
+		sheet_queue_respan (sheet, 0, SHEET_MAX_ROWS-1);
+	} else {
+		workbook_sheet_detach (book, sheet, TRUE);
 		gnumeric_error_read (COMMAND_CONTEXT (context),
 			_("Parse error while trying to parse data into sheet"));
-		return;
 	}
+
 	stf_parse_options_free (po);
-
-	workbook_recalc (book);
-	sheet_queue_respan (sheet, 0, SHEET_MAX_ROWS-1);
-
-	g_free (name);
-	g_free (data);
-}
-
-static gboolean
-stf_read_default_probe (GnmFileOpener const *fo, GsfInput *input, FileProbeLevel pl)
-{
-	guint8 const *data;
-	gsf_off_t remain;
-	int len;
-
-	if (pl < FILE_PROBE_CONTENT)
-		return FALSE;
-
-	if (gsf_input_seek (input, 0, G_SEEK_SET))
-		return FALSE;
-	remain = gsf_input_remaining (input);
-	len = MIN (remain, STF_PROBE_SIZE);
-	data = gsf_input_read (input, len, NULL);
-	return (data != NULL) && (stf_parse_is_valid_data (data, len) == NULL);
+	g_free (utf8data);
 }
 
 /***********************************************************************************/
@@ -531,15 +426,31 @@ stf_write_workbook (GnmFileSaver const *fs, IOContext *context,
 		gnumeric_io_error_unknown (context);
 }
 
+static gboolean
+csv_tsv_probe (GnmFileOpener const *fo, GsfInput *input, FileProbeLevel pl)
+{
+	if (pl == FILE_PROBE_FILE_NAME) {
+		char const *name = gsf_input_name (input);
+		if (name == NULL)
+			return FALSE;
+		name = gsf_extension_pointer (name);
+		return (name != NULL &&
+		        (g_ascii_strcasecmp (name, "csv") == 0 ||
+			 g_ascii_strcasecmp (name, "tsv") == 0));
+	} else if (pl == FILE_PROBE_CONTENT)
+		return TRUE;
+	return FALSE;
+}
+
 void
 stf_init (void)
 {
 	gnm_file_opener_register (gnm_file_opener_new_with_enc (
 		"Gnumeric_stf:stf_csvtab",
 		_("Comma or tab separated files (CSV/TSV)"),
-		stf_read_default_probe, stf_read_workbook_auto_csvtab), 0);
+		csv_tsv_probe, stf_read_workbook_auto_csvtab), 0);
 	gnm_file_opener_register (gnm_file_opener_new_with_enc (
-		"Gnumeric_stf:stf_druid", 
+		"Gnumeric_stf:stf_druid",
 		_("Text import (configurable)"),
 		NULL, stf_read_workbook), 0);
 	gnm_file_saver_register (gnm_file_saver_new (
