@@ -29,9 +29,9 @@
 static GHashTable *style_font_hash;
 static GHashTable *style_font_negative_hash;
 
-StyleFont *gnumeric_default_font;
-StyleFont *gnumeric_default_bold_font;
-StyleFont *gnumeric_default_italic_font;
+double gnumeric_default_font_width;
+static char *gnumeric_default_font_name;
+static double gnumeric_default_font_size;
 
 /**
  * get_substitute_font
@@ -66,7 +66,7 @@ get_substitute_font (gchar const *fontname)
 	return NULL;
 }
 
-int
+static int
 style_font_string_width (StyleFont const *font, char const *str)
 {
 	int w;
@@ -104,19 +104,20 @@ calc_font_width (StyleFont const *font, char const *teststr)
 }
 
 
-StyleFont *
-style_font_new_simple (char const *font_name, double size_pts, double scale,
+static StyleFont *
+style_font_new_simple (PangoContext *context,
+		       char const *font_name, double size_pts, double scale,
 		       gboolean bold, gboolean italic)
 {
 	StyleFont *font;
 	StyleFont key;
 
 	if (font_name == NULL) {
-		g_warning ("font_name == NULL, Using %s", DEFAULT_FONT);
+		g_warning ("font_name == NULL, using %s", DEFAULT_FONT);
 		font_name = DEFAULT_FONT;
 	}
-	if (size_pts == 0) {
-		g_warning ("font_name == NULL, Using %f", DEFAULT_SIZE);
+	if (size_pts <= 0) {
+		g_warning ("font_size <= 0, using %f", DEFAULT_SIZE);
 		size_pts = DEFAULT_SIZE;
 	}
 
@@ -144,31 +145,31 @@ style_font_new_simple (char const *font_name, double size_pts, double scale,
 		/* One reference for the cache, one for the caller. */
 		font->ref_count = 2;
 
-		font->pango.context = gdk_pango_context_get ();
-		pango_context_set_language (font->pango.context, gtk_get_default_language ());
-
-		desc = pango_context_get_font_description (font->pango.context);
+		g_object_ref (context);
+		font->pango.context = context;
+		desc = pango_context_get_font_description (context);
 		pango_font_description_set_family (desc, font_name);
 		pango_font_description_set_weight (desc,
 			bold ? PANGO_WEIGHT_BOLD : PANGO_WEIGHT_NORMAL);
 		pango_font_description_set_style (desc,
 			italic ? PANGO_STYLE_ITALIC : PANGO_STYLE_NORMAL);
+		/* FIXME: set scale separately?  */
 		pango_font_description_set_size (desc,
-			size_pts*scale * PANGO_SCALE);
+						 size_pts * scale *
+						 PANGO_SCALE);
 
-		font->pango.font = pango_context_load_font (font->pango.context,
-							    desc);
+		font->pango.font = pango_context_load_font (context, desc);
 		if (font->pango.font == NULL) {
 			/* if we fail, try to be smart and map to something similar */
 			char const *sub = get_substitute_font (font_name);
 			if (sub != NULL) {
 				pango_font_description_set_family (desc, font_name);
-				font->pango.font = pango_context_load_font (font->pango.context,
+				font->pango.font = pango_context_load_font (context,
 									    desc);
 			}
 
 			if (font->pango.font == NULL) {
-				g_object_unref (G_OBJECT (font->pango.context));
+				g_object_unref (context);
 				font->pango.context = NULL;
 				font->pango.font_descr = NULL;
 				g_hash_table_insert (style_font_negative_hash,
@@ -179,10 +180,7 @@ style_font_new_simple (char const *font_name, double size_pts, double scale,
 
 		font->pango.font_descr = pango_font_describe (font->pango.font);
 
-		gdk_pango_context_set_colormap (font->pango.context,
-						gtk_widget_get_default_colormap ());
-
-		font->pango.layout  = pango_layout_new (font->pango.context);
+		font->pango.layout  = pango_layout_new (context);
 		pango_layout_set_font_description (font->pango.layout,
 						   font->pango.font_descr);
 
@@ -226,35 +224,45 @@ style_font_new_simple (char const *font_name, double size_pts, double scale,
 }
 
 StyleFont *
-style_font_new (char const *font_name, double size_pts, double scale,
+style_font_new (PangoContext *context,
+		char const *font_name, double size_pts, double scale,
 		gboolean bold, gboolean italic)
 {
 	StyleFont *font;
 
 	g_return_val_if_fail (font_name != NULL, NULL);
-	g_return_val_if_fail (size_pts != 0, NULL);
+	g_return_val_if_fail (size_pts > 0, NULL);
 
-	font = style_font_new_simple (font_name, size_pts, scale, bold, italic);
-	if (!font){
-		if (bold)
-			font = gnumeric_default_bold_font;
-		else if (italic)
-			font = gnumeric_default_italic_font;
-		else
-			font = gnumeric_default_font;
-		style_font_ref (font);
-	}
+	font = style_font_new_simple (context, font_name, size_pts,
+				      scale, bold, italic);
+	if (font) return font;
 
-	return font;
-}
+	font_name = gnumeric_default_font_name;
+	font = style_font_new_simple (context, font_name, size_pts,
+				      scale, bold, italic);
+	if (font) return font;
 
-int
-style_font_get_height (StyleFont const *sf)
-{
-	g_return_val_if_fail (sf != NULL, 0);
+	size_pts = gnumeric_default_font_size;
+	font = style_font_new_simple (context, font_name, size_pts,
+				      scale, bold, italic);
+	if (font) return font;
 
-	return PANGO_PIXELS(pango_font_metrics_get_ascent(sf->pango.metrics) +
-	       pango_font_metrics_get_descent(sf->pango.metrics));
+	bold = FALSE;
+	font = style_font_new_simple (context, font_name, size_pts,
+				      scale, bold, italic);
+	if (font) return font;
+
+	italic = FALSE;
+	font = style_font_new_simple (context, font_name, size_pts,
+				      scale, bold, italic);
+	if (font) return font;
+
+	/*
+	 * This should not be possible to reach as we have reverted all the way
+	 * back to the default font.
+	 */
+	g_assert_not_reached ();
+	abort ();
 }
 
 void
@@ -352,46 +360,57 @@ static void
 font_init (void)
 {
 	GConfClient *client = application_get_gconf_client ();
-	char *font_name =  gconf_client_get_string (client,
-					  GCONF_DEFAULT_FONT, NULL);
-	double font_size = gconf_client_get_float (client,
-						   GCONF_DEFAULT_SIZE, NULL);
-	gnumeric_default_font = NULL;
+	GdkScreen *screen;
+	PangoContext *context;
+	StyleFont *gnumeric_default_font = NULL;
 
-	if (font_name && font_size >= 1)
-		gnumeric_default_font = style_font_new_simple (font_name, font_size,
-							       1., FALSE, FALSE);
+	gnumeric_default_font_name =
+		gconf_client_get_string (client,
+					 GCONF_DEFAULT_FONT,
+					 NULL);
+	gnumeric_default_font_size =
+		gconf_client_get_float (client,
+					GCONF_DEFAULT_SIZE,
+					NULL);
+
+	screen = gdk_screen_get_default ();
+	context = gdk_pango_context_get_for_screen (screen);
+	pango_context_set_language (context, gtk_get_default_language ());
+	gdk_pango_context_set_colormap (context,
+					gdk_screen_get_default_colormap (screen));
+
+	if (gnumeric_default_font_name && gnumeric_default_font_size >= 1)
+		gnumeric_default_font =
+			style_font_new_simple (context,
+					       gnumeric_default_font_name,
+					       gnumeric_default_font_size,
+					       1., FALSE, FALSE);
 	if (!gnumeric_default_font) {
 		g_warning ("Configured default font not available, trying fallback...");
-		gnumeric_default_font = style_font_new_simple (DEFAULT_FONT, DEFAULT_SIZE,
-							       1., FALSE, FALSE);
+		gnumeric_default_font =
+			style_font_new_simple (context,
+					       DEFAULT_FONT, DEFAULT_SIZE,
+					       1., FALSE, FALSE);
+		g_free (gnumeric_default_font_name);
+		gnumeric_default_font_name = g_strdup (DEFAULT_FONT);
+		gnumeric_default_font_size = DEFAULT_SIZE;
 	}
 
+	g_object_unref (context);
+
 	if (!gnumeric_default_font)
 		exit (1);
 
-	gnumeric_default_bold_font = style_font_new_simple (
-		font_name, font_size, 1., TRUE, FALSE);
-	if (gnumeric_default_bold_font == NULL){
-	    gnumeric_default_bold_font = gnumeric_default_font;
-	    style_font_ref (gnumeric_default_bold_font);
-	}
-
-	gnumeric_default_italic_font = style_font_new_simple (
-		font_name, font_size, 1., FALSE, TRUE);
-	if (gnumeric_default_italic_font == NULL){
-		gnumeric_default_italic_font = gnumeric_default_font;
-		style_font_ref (gnumeric_default_italic_font);
-	}
-	g_free (font_name);
+	gnumeric_default_font_width = gnumeric_default_font->approx_width.pts.digit;
+	style_font_unref (gnumeric_default_font);
 }
 
 void
 style_init (void)
 {
 	number_format_init ();
-	style_font_hash   = g_hash_table_new (style_font_hash_func,
-					      style_font_equal);
+	style_font_hash = g_hash_table_new (style_font_hash_func,
+					    style_font_equal);
 	style_font_negative_hash = g_hash_table_new (style_font_hash_func,
 						     style_font_equal);
 
@@ -422,30 +441,8 @@ list_cached_fonts (gpointer key, gpointer value, gpointer user_data)
 void
 style_shutdown (void)
 {
-	if (gnumeric_default_font != gnumeric_default_bold_font &&
-	    gnumeric_default_bold_font->ref_count != 2) {
-		g_warning ("Default bold font has %d references.  It should have two.",
-			   gnumeric_default_bold_font->ref_count);
-	}
-	style_font_unref (gnumeric_default_bold_font);
-	gnumeric_default_bold_font = NULL;
-
-	if (gnumeric_default_font != gnumeric_default_italic_font &&
-	    gnumeric_default_italic_font->ref_count != 2) {
-		g_warning ("Default italic font has %d references.  It should have two.",
-			   gnumeric_default_italic_font->ref_count);
-	}
-	style_font_unref (gnumeric_default_italic_font);
-	gnumeric_default_italic_font = NULL;
-
-	/* At this point, even if we had bold == normal (etc), we should
-	   have exactly two references to default.  */
-	if (gnumeric_default_font->ref_count != 2) {
-		g_warning ("Default font has %d references.  It should have two.",
-			   gnumeric_default_font->ref_count);
-	}
-	style_font_unref (gnumeric_default_font);
-	gnumeric_default_font = NULL;
+	g_free (gnumeric_default_font_name);
+	gnumeric_default_font_name = NULL;
 
 	number_format_shutdown ();
 	{
