@@ -5,7 +5,7 @@
 #include <libgnomeprint/gnome-font.h>
 #include "gnumeric.h"
 
-#define DEFAULT_FONT "Sans"
+#define DEFAULT_FONT "Sans Regular"
 #define DEFAULT_SIZE 10.0
 #define GCONF_DEFAULT_FONT "/apps/gnumeric/core/defaultfont/name"
 #define GCONF_DEFAULT_SIZE "/apps/gnumeric/core/defaultfont/size"
@@ -67,10 +67,6 @@ StyleFont     *style_font_new         (PangoContext *context,
 				       const char *font_name,
 				       double size_pts, double scale,
 				       gboolean bold, gboolean italic);
-StyleFont     *style_font_new_simple  (PangoContext *context,
-				       const char *font_name,
-				       double size_pts, double scale,
-				       gboolean bold, gboolean italic);
 void style_font_ref          (StyleFont *sf);
 void style_font_unref        (StyleFont *sf);
 
