@@ -46,6 +46,9 @@
 #include <unistd.h>
 #include <sys/times.h>
 
+
+extern const gchar *solver_max_time_err;
+
 /* ------------------------------------------------------------------------- */
 
 
@@ -57,6 +60,7 @@ solver_param_new (void)
 	res->options.model_type          = SolverLPModel;
 	res->options.assume_non_negative = TRUE;
 	res->options.algorithm           = GLPKSimplex;
+	res->options.scenario_name       = g_strdup ("Optimal");
 	res->input_entry_str             = g_strdup ("");
 	res->problem_type                = SolverMaximize;
 	res->constraints                 = NULL;
@@ -71,6 +75,7 @@ void
 solver_param_destroy (SolverParameters *sp)
 {
 	g_free (sp->input_entry_str);
+	g_free (sp->options.scenario_name);
 	g_free (sp);
 }
 
@@ -79,14 +84,14 @@ solver_results_init (const SolverParameters *sp)
 {
         SolverResults *res     = g_new (SolverResults, 1);
 
-	res->optimal_values    = g_new (gnum_float,  sp->n_variables);
-	res->original_values   = g_new (gnum_float,  sp->n_variables);
+	res->optimal_values    = g_new (gnm_float,  sp->n_variables);
+	res->original_values   = g_new (gnm_float,  sp->n_variables);
 	res->variable_names    = g_new0 (gchar *,    sp->n_variables);
 	res->constraint_names  = g_new0 (gchar *,    sp->n_total_constraints);
-	res->shadow_prizes     = g_new0 (gnum_float, sp->n_total_constraints);
-	res->slack             = g_new0 (gnum_float, sp->n_total_constraints);
-	res->lhs               = g_new0 (gnum_float, sp->n_total_constraints);
-	res->rhs               = g_new0 (gnum_float, sp->n_total_constraints);
+	res->shadow_prizes     = g_new0 (gnm_float, sp->n_total_constraints);
+	res->slack             = g_new0 (gnm_float, sp->n_total_constraints);
+	res->lhs               = g_new0 (gnm_float, sp->n_total_constraints);
+	res->rhs               = g_new0 (gnm_float, sp->n_total_constraints);
 	res->n_variables       = sp->n_variables;
 	res->n_constraints     = sp->n_constraints;
 	res->n_nonzeros_in_obj = 0;
@@ -103,9 +108,9 @@ solver_results_init (const SolverParameters *sp)
 	res->constr_coeff      = NULL;
 	res->limits            = NULL;
 	res->constr_allowable_increase  =
-	        g_new0 (gnum_float, sp->n_total_constraints);
+	        g_new0 (gnm_float, sp->n_total_constraints);
 	res->constr_allowable_decrease =
-	        g_new0 (gnum_float, sp->n_total_constraints);
+	        g_new0 (gnm_float, sp->n_total_constraints);
 
 	return res;
 }
@@ -191,7 +196,7 @@ write_constraint_str (int lhs_col, int lhs_row, int rhs_col,
 		      int rhs_row, SolverConstraintType type,
 		      int cols, int rows)
 {
-	GString    *buf = g_string_new ("");
+	GString    *buf = g_string_new (NULL);
 	const char *type_str[] = { "<=", ">=", "=", "Int", "Bool" };
 	char       *result;
 
@@ -238,10 +243,10 @@ write_constraint_str (int lhs_col, int lhs_row, int rhs_col,
  * returns the coefficent of the first variable of the objective
  * function.
  */
-static inline gnum_float
-get_lp_coeff (Cell *target, Cell *change, gnum_float *x0)
+static inline gnm_float
+get_lp_coeff (Cell *target, Cell *change, gnm_float *x0)
 {
-        gnum_float tmp = *x0;
+        gnm_float tmp = *x0;
 
 	cell_set_value (change, value_new_float (1.0));
 	cell_queue_recalc (change);
@@ -306,24 +311,6 @@ restore_original_values (SolverResults *res)
 /************************************************************************
  */
 
-#if 0
-static void
-callback (int iter, gnum_float *x, gnum_float bv, gnum_float cx, int n,
-	  void *data)
-{
-        int     i;
-
-	printf ("Iteration=%3d ", iter + 1);
-	printf ("bv=%9.4" GNUM_FORMAT_f
-		" cx=%9.4" GNUM_FORMAT_f
-		" gap=%9.4" GNUM_FORMAT_f "\n",
-		bv, cx, gnumabs (bv - cx));
-	for (i = 0; i < n; i++)
-	        printf ("%8.4" GNUM_FORMAT_f " ", x[i]);
-        printf ("\n");
-}
-#endif
-
 static int
 get_col_nbr (SolverResults *res, CellPos *pos)
 {
@@ -358,11 +345,11 @@ clear_input_vars (int n_variables, SolverResults *res)
 static SolverProgram
 lp_qp_solver_init (Sheet *sheet, const SolverParameters *param,
 		   SolverResults *res, const SolverLPAlgorithm *alg,
-		   gnum_float start_time, gchar **errmsg)
+		   gnm_float start_time, GTimeVal start, const gchar **errmsg)
 {
         SolverProgram     program;
 	Cell              *target;
-	gnum_float        x, x0, base;
+	gnm_float         x, x0, base;
 	int               i, n, ind;
 
 	/* Initialize the SolverProgram structure. */
@@ -398,6 +385,8 @@ lp_qp_solver_init (Sheet *sheet, const SolverParameters *param,
 	/* Add constraints. */
 	for (i = ind = 0; i < param->n_total_constraints; i++) {
 	        SolverConstraint *c = solver_get_constraint (res, i);
+		GTimeVal cur_time;
+
 		target = sheet_cell_get (sheet, c->lhs.col, c->lhs.row);
 
 		/* Check that LHS is a number type. */
@@ -452,6 +441,16 @@ lp_qp_solver_init (Sheet *sheet, const SolverParameters *param,
 		x = value_get_as_float (target->value);
 		alg->set_constr_fn (program, ind, c->type, x);
 		ind++;
+
+		/* Check that max time has not elapsed. */
+		g_get_current_time (&cur_time);
+		if (cur_time.tv_sec - start.tv_sec >
+		    param->options.max_time_sec) {
+			*errmsg = solver_max_time_err;
+			solver_results_free (res);
+			return NULL;
+		}
+		
 	}
 
 	/* Set up the problem type. */
@@ -498,7 +497,7 @@ static gboolean
 check_program_definition_failures (Sheet            *sheet,
 				   SolverParameters *param,
 				   SolverResults    **res,
-				   gchar            **errmsg)
+				   const gchar      **errmsg)
 {
 	GSList           *inputs;
 	GSList           *c;
@@ -583,11 +582,11 @@ check_program_definition_failures (Sheet            *sheet,
 	(*res)->param = param;
 	(*res)->input_cells_array = input_cells_array;
 	(*res)->constraints_array = constraints_array;
-	(*res)->obj_coeff = g_new0 (gnum_float, param->n_variables);
+	(*res)->obj_coeff = g_new0 (gnm_float, param->n_variables);
 
-	(*res)->constr_coeff = g_new0 (gnum_float *, param->n_total_constraints);
+	(*res)->constr_coeff = g_new0 (gnm_float *, param->n_total_constraints);
 	for (i = 0; i < param->n_total_constraints; i++)
-	        (*res)->constr_coeff[i] = g_new0 (gnum_float,
+	        (*res)->constr_coeff[i] = g_new0 (gnm_float,
 						  param->n_variables);
 	(*res)->limits = g_new (SolverLimits, param->n_variables);
 
@@ -596,7 +595,7 @@ check_program_definition_failures (Sheet            *sheet,
 
 static SolverResults *
 solver_run (WorkbookControl *wbc, Sheet *sheet,
-	    const SolverLPAlgorithm *alg, gchar **errmsg)
+	    const SolverLPAlgorithm *alg, const gchar **errmsg)
 {
 	SolverParameters  *param = sheet->solver_parameters;
 	SolverProgram     program;
@@ -609,24 +608,25 @@ solver_run (WorkbookControl *wbc, Sheet *sheet,
 	if (check_program_definition_failures (sheet, param, &res, errmsg))
 	        return NULL;
 
-	res->time_user   = - buf.tms_utime / (gnum_float) sysconf (_SC_CLK_TCK);
-	res->time_system = - buf.tms_stime / (gnum_float) sysconf (_SC_CLK_TCK);
+	res->time_user   = - buf.tms_utime / (gnm_float) sysconf (_SC_CLK_TCK);
+	res->time_system = - buf.tms_stime / (gnm_float) sysconf (_SC_CLK_TCK);
 	res->time_real   = - (start.tv_sec +
-			      start.tv_usec / (gnum_float) G_USEC_PER_SEC);
+			      start.tv_usec / (gnm_float) G_USEC_PER_SEC);
 	save_original_values (res, param, sheet);
 
 	program              = lp_qp_solver_init (sheet, param, res, alg, 
-						  -res->time_real, errmsg);
+						  -res->time_real, start,
+						  errmsg);
 	if (program == NULL)
 	        return NULL;
 
         res->status = alg->solve_fn (program);
 	g_get_current_time (&end);
 	times (&buf);
-	res->time_user   += buf.tms_utime / (gnum_float) sysconf (_SC_CLK_TCK);
-	res->time_system += buf.tms_stime / (gnum_float) sysconf (_SC_CLK_TCK);
+	res->time_user   += buf.tms_utime / (gnm_float) sysconf (_SC_CLK_TCK);
+	res->time_system += buf.tms_stime / (gnm_float) sysconf (_SC_CLK_TCK);
 	res->time_real   += end.tv_sec + end.tv_usec /
-	        (gnum_float) G_USEC_PER_SEC;
+	        (gnm_float) G_USEC_PER_SEC;
 	res->n_iterations = alg->get_iterations_fn (program);
 
 	solver_prepare_reports (program, res, sheet);
@@ -644,7 +644,7 @@ solver_run (WorkbookControl *wbc, Sheet *sheet,
 }
 
 SolverResults *
-solver (WorkbookControl *wbc, Sheet *sheet, gchar **errmsg)
+solver (WorkbookControl *wbc, Sheet *sheet, const gchar **errmsg)
 {
 	const SolverLPAlgorithm *alg = NULL;
 	SolverParameters  *param = sheet->solver_parameters;
@@ -684,7 +684,9 @@ solver_lp_copy (const SolverParameters *src_param, Sheet *new_sheet)
 	g_free (dst_param->input_entry_str);
 	dst_param->input_entry_str = g_strdup (src_param->input_entry_str);
 
+	g_free (dst_param->options.scenario_name);
 	dst_param->options = src_param->options;
+	dst_param->options.scenario_name = g_strdup (src_param->options.scenario_name);
 	/* Had there been any non-scalar options, we'd copy them here.  */
 
 	/* Copy the constraints */
